@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2000, 2015, Oracle and/or its affiliates.
-   Copyright (c) 2011, 2022, MariaDB
+   Copyright (c) 2011, 2017, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -63,6 +63,9 @@ static uint     opt_mysql_port= 0, opt_protocol= 0;
 static char * opt_mysql_unix_port=0;
 static char *opt_plugin_dir= 0, *opt_default_auth= 0;
 static longlong opt_ignore_lines= -1;
+
+static uint protocol_to_force= MYSQL_PROTOCOL_DEFAULT;
+
 #include <sslopt-vars.h>
 
 static char **argv_to_free;
@@ -221,19 +224,27 @@ file. The SQL command 'LOAD DATA INFILE' is used to import the rows.\n");
 
 
 static my_bool
-get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
-	       char *argument)
+get_one_option(const struct my_option *opt, const char *argument,
+               const char *filename)
 {
-  switch(optid) {
+  /* Track when protocol is set via CLI to not force overrides */
+  static my_bool ignore_protocol_override = FALSE;
+
+  switch(opt->id) {
   case 'p':
     if (argument == disabled_my_option)
       argument= (char*) "";			/* Don't require password */
     if (argument)
     {
-      char *start=argument;
+      /*
+        One should not really change the argument, but we make an
+        exception for passwords
+      */
+      char *start= (char*) argument;
       my_free(opt_password);
-      opt_password=my_strdup(argument,MYF(MY_FAE));
-      while (*argument) *argument++= 'x';		/* Destroy argument */
+      opt_password=my_strdup(PSI_NOT_INSTRUMENTED, argument,MYF(MY_FAE));
+      while (*argument)
+        *(char*) argument++= 'x';               /* Destroy argument */
       if (*start)
 	start[1]=0;				/* Cut length of argument */
       tty_password= 0;
@@ -245,6 +256,14 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
   case 'W':
     opt_protocol = MYSQL_PROTOCOL_PIPE;
     opt_local_file=1;
+
+    /* Prioritize pipe if explicit via command line */
+    if (filename[0] == '\0')
+    {
+      ignore_protocol_override = TRUE;
+      protocol_to_force = MYSQL_PROTOCOL_DEFAULT;
+    }
+
     break;
 #endif
   case OPT_MYSQL_PROTOCOL:
@@ -254,6 +273,46 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
       sf_leaking_memory= 1; /* no memory leak reports here */
       exit(1);
     }
+
+    /* Specification of protocol via CLI trumps implicit overrides */
+    if (filename[0] == '\0')
+    {
+      ignore_protocol_override = TRUE;
+      protocol_to_force = MYSQL_PROTOCOL_DEFAULT;
+    }
+
+    break;
+  case 'P':
+    /* If port and socket are set, fall back to default behavior */
+    if (protocol_to_force == SOCKET_PROTOCOL_TO_FORCE)
+    {
+      ignore_protocol_override = TRUE;
+      protocol_to_force = MYSQL_PROTOCOL_DEFAULT;
+    }
+
+    /* If port is set via CLI, try to force protocol to TCP */
+    if (filename[0] == '\0' &&
+        !ignore_protocol_override &&
+        protocol_to_force == MYSQL_PROTOCOL_DEFAULT)
+    {
+      protocol_to_force = MYSQL_PROTOCOL_TCP;
+    }
+    break;
+  case 'S':
+    /* If port and socket are set, fall back to default behavior */
+    if (protocol_to_force == MYSQL_PROTOCOL_TCP)
+    {
+      ignore_protocol_override = TRUE;
+      protocol_to_force = MYSQL_PROTOCOL_DEFAULT;
+    }
+
+    /* Prioritize socket if set via command line */
+    if (filename[0] == '\0' &&
+        !ignore_protocol_override &&
+        protocol_to_force == MYSQL_PROTOCOL_DEFAULT)
+    {
+      protocol_to_force = SOCKET_PROTOCOL_TO_FORCE;
+    }
     break;
   case '#':
     DBUG_PUSH(argument ? argument : "d:t:o");
@@ -323,7 +382,11 @@ static int write_to_table(char *filename, MYSQL *mysql)
   {
     if (verbose)
       fprintf(stdout, "Deleting the old data from table %s\n", tablename);
+#ifdef HAVE_SNPRINTF
     snprintf(sql_statement, FN_REFLEN*16+256, "DELETE FROM %s", tablename);
+#else
+    sprintf(sql_statement, "DELETE FROM %s", tablename);
+#endif
     if (mysql_query(mysql, sql_statement))
     {
       db_error_with_table(mysql, tablename);
@@ -510,15 +573,13 @@ static void safe_exit(int error, MYSQL *mysql)
   if (mysql)
     mysql_close(mysql);
 
+  mysql_library_end();
+  free_defaults(argv_to_free);
+  my_free(opt_password);
   if (error)
     sf_leaking_memory= 1; /* dirty exit, some threads are still running */
   else
-  {
-    mysql_library_end();
-    free_defaults(argv_to_free);
-    my_free(opt_password);
     my_end(my_end_arg); /* clean exit */
-  }
   exit(error);
 }
 
@@ -562,7 +623,7 @@ static char *add_load_option(char *ptr, const char *object,
 /*
 ** Allow the user to specify field terminator strings like:
 ** "'", "\", "\\" (escaped backslash), "\t" (tab), "\n" (newline)
-** This is done by doubling ' and add a end -\ if needed to avoid
+** This is done by doubleing ' and add a end -\ if needed to avoid
 ** syntax errors from the SQL parser.
 */ 
 
@@ -638,6 +699,9 @@ int main(int argc, char **argv)
   MY_INIT(argv[0]);
   sf_leaking_memory=1; /* don't report memory leaks on early exits */
 
+  /* We need to know if protocol-related options originate from CLI args */
+  my_defaults_mark_files = TRUE;
+
   load_defaults_or_exit("my", load_default_groups, &argc, &argv);
   /* argv is changed in the program */
   argv_to_free= argv;
@@ -646,6 +710,14 @@ int main(int argc, char **argv)
     free_defaults(argv_to_free);
     return(1);
   }
+
+  /* Command line options override configured protocol */
+  if (protocol_to_force > MYSQL_PROTOCOL_DEFAULT
+      && protocol_to_force != opt_protocol)
+  {
+    warn_protocol_override(current_host, &opt_protocol, protocol_to_force);
+  }
+
   sf_leaking_memory=0; /* from now on we cleanup properly */
 
   if (opt_use_threads && !lock_tables)
@@ -670,9 +742,8 @@ int main(int argc, char **argv)
       table_count++;
     argv= save_argv;
 
-    if (!(worker_threads= (pthread_t*) my_malloc(table_count *
-                                                 sizeof(*worker_threads),
-                                                 MYF(0))))
+    if (!(worker_threads= (pthread_t*) my_malloc(PSI_NOT_INSTRUMENTED,
+                               table_count * sizeof(*worker_threads), MYF(0))))
       return -2;
 
     for (; *argv != NULL; argv++) /* Loop through tables */
