@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2005, 2016, Oracle and/or its affiliates. All rights reserved.
-   Copyright (c) 2014, 2022, MariaDB Corporation.
+   Copyright (c) 2014, 2021, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -53,12 +53,6 @@ The parts not included are excluded by #ifndef UNIV_INNOCHECKSUM. */
 
 #include <string.h>
 
-#ifdef UNIV_NONINL
-# include "fsp0fsp.inl"
-# include "mach0data.inl"
-# include "ut0rnd.inl"
-#endif
-
 #ifndef PRIuMAX
 #define PRIuMAX   "llu"
 #endif
@@ -66,26 +60,23 @@ The parts not included are excluded by #ifndef UNIV_INNOCHECKSUM. */
 /* Global variables */
 static bool			verbose;
 static bool			just_count;
-static uint32_t			start_page;
-static uint32_t			end_page;
-static uint32_t			do_page;
+static unsigned long long	start_page;
+static unsigned long long	end_page;
+static unsigned long long	do_page;
 static bool			use_end_page;
 static bool			do_one_page;
 static my_bool do_leaf;
 static my_bool per_page_details;
 static ulint n_merge;
-extern ulong			srv_checksum_algorithm;
 static ulint physical_page_size;  /* Page size in bytes on disk. */
 ulong srv_page_size;
 ulong srv_page_size_shift;
 /* Current page number (0 based). */
-uint32_t		cur_page_num;
+unsigned long long		cur_page_num;
 /* Current space. */
-uint32_t		cur_space;
+unsigned long long		cur_space;
 /* Skip the checksum verification. */
 static bool			no_check;
-/* Enabled for strict checksum verification. */
-bool				strict_verify = 0;
 /* Enabled for rewrite checksum. */
 static bool			do_write;
 /* Mismatches count allowed (0 by default). */
@@ -103,27 +94,22 @@ FILE*				log_file = NULL;
 /* Enabled for log write option. */
 static bool			is_log_enabled = false;
 
-static byte field_ref_zero_buf[UNIV_PAGE_SIZE_MAX];
-const byte *field_ref_zero = field_ref_zero_buf;
-
 #ifndef _WIN32
 /* advisory lock for non-window system. */
 struct flock			lk;
 #endif /* _WIN32 */
 
-/* Strict check algorithm name. */
-static ulong			strict_check;
-/* Rewrite checksum algorithm name. */
-static ulong			write_check;
-
 /* Innodb page type. */
 struct innodb_page_type {
 	int n_undo_state_active;
 	int n_undo_state_cached;
+	int n_undo_state_to_free;
 	int n_undo_state_to_purge;
 	int n_undo_state_prepared;
 	int n_undo_state_other;
-	int n_undo;
+	int n_undo_insert;
+	int n_undo_update;
+	int n_undo_other;
 	int n_fil_page_index;
 	int n_fil_page_undo_log;
 	int n_fil_page_inode;
@@ -142,24 +128,6 @@ struct innodb_page_type {
 	int n_fil_page_type_page_compressed_encrypted;
 } page_type;
 
-/* Possible values for "--strict-check" for strictly verify checksum
-and "--write" for rewrite checksum. */
-static const char *innochecksum_algorithms[] = {
-	"crc32",
-	"crc32",
-	"innodb",
-	"innodb",
-	"none",
-	"none",
-	NullS
-};
-
-/* Used to define an enumerate type of the "innochecksum algorithm". */
-static TYPELIB innochecksum_algorithms_typelib = {
-	array_elements(innochecksum_algorithms)-1,"",
-	innochecksum_algorithms, NULL
-};
-
 #define SIZE_RANGES_FOR_PAGE 10
 #define NUM_RETRIES 3
 #define DEFAULT_RETRY_DELAY 1000000
@@ -471,11 +439,11 @@ is_page_corrupted(
 	/* enable if page is corrupted. */
 	bool is_corrupted;
 	/* use to store LSN values. */
-	uint32_t logseq;
-	uint32_t logseqfield;
-	const uint16_t page_type = mach_read_from_2(buf+FIL_PAGE_TYPE);
-	uint32_t key_version = buf_page_get_key_version(buf, flags);
-	uint32_t space_id = mach_read_from_4(
+	ulint logseq;
+	ulint logseqfield;
+	const uint16_t page_type = fil_page_get_type(buf);
+	uint key_version = buf_page_get_key_version(buf, flags);
+	ulint space_id = mach_read_from_4(
 		buf + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID);
 	ulint zip_size = fil_space_t::zip_size(flags);
 	ulint is_compressed = fil_space_t::is_compressed(flags);
@@ -493,8 +461,8 @@ is_page_corrupted(
 
 		if (is_log_enabled) {
 			fprintf(log_file,
-				"page id mismatch space::" UINT32PF
-				" page::" UINT32PF " \n",
+				"page id mismatch space::" ULINTPF
+				" page::%llu \n",
 				space_id, cur_page_num);
 		}
 
@@ -523,14 +491,13 @@ is_page_corrupted(
 
 		if (is_log_enabled) {
 			fprintf(log_file,
-				"space::" UINT32PF " page::" UINT32PF
-				"; log sequence number:first = " UINT32PF
-				"; second = " UINT32PF "\n",
+				"space::" ULINTPF " page::%llu"
+				"; log sequence number:first = " ULINTPF
+				"; second = " ULINTPF "\n",
 				space_id, cur_page_num, logseq, logseqfield);
 			if (logseq != logseqfield) {
 				fprintf(log_file,
-					"Fail; space::" UINT32PF
-					" page::" UINT32PF
+					"Fail; space::" ULINTPF " page::%llu"
 					" invalid (fails log "
 					"sequence number check)\n",
 					space_id, cur_page_num);
@@ -554,9 +521,9 @@ is_page_corrupted(
 
 		if (is_corrupted && log_file) {
 			fprintf(log_file,
-				"[page id: space=" UINT32PF
-				", page_number=" UINT32PF "] may be corrupted;"
-				" key_version=" UINT32PF "\n",
+				"[page id: space=" ULINTPF
+				", page_number=%llu] may be corrupted;"
+				" key_version=%u\n",
 				space_id, cur_page_num, key_version);
 		}
 	} else {
@@ -653,15 +620,14 @@ static bool update_checksum(byte* page, ulint flags)
 	}
 
 	if (iscompressed) {
-		/* page is compressed */
-		checksum = page_zip_calc_checksum(
-			page, physical_page_size,
-			static_cast<srv_checksum_algorithm_t>(write_check));
+		/* ROW_FORMAT=COMPRESSED */
+		checksum = page_zip_calc_checksum(page, physical_page_size,
+						  false);
 
 		mach_write_to_4(page + FIL_PAGE_SPACE_OR_CHKSUM, checksum);
 		if (is_log_enabled) {
-			fprintf(log_file, "page::" UINT32PF "; Updated checksum ="
-				" " UINT32PF "\n", cur_page_num, checksum);
+			fprintf(log_file, "page::%llu; Updated checksum ="
+				" %u\n", cur_page_num, checksum);
 		}
 
 	} else if (use_full_crc32) {
@@ -672,7 +638,7 @@ static bool update_checksum(byte* page, ulint flags)
 		if (mach_read_from_4(c) == checksum) return false;
 		mach_write_to_4(c, checksum);
 		if (is_log_enabled) {
-			fprintf(log_file, "page::" UINT32PF "; Updated checksum"
+			fprintf(log_file, "page::%llu; Updated checksum"
 				" = %u\n", cur_page_num, checksum);
 		}
 		return true;
@@ -680,50 +646,16 @@ static bool update_checksum(byte* page, ulint flags)
 		/* page is uncompressed. */
 
 		/* Store the new formula checksum */
-		switch ((srv_checksum_algorithm_t) write_check) {
-
-		case SRV_CHECKSUM_ALGORITHM_FULL_CRC32:
-		case SRV_CHECKSUM_ALGORITHM_STRICT_FULL_CRC32:
-		case SRV_CHECKSUM_ALGORITHM_CRC32:
-		case SRV_CHECKSUM_ALGORITHM_STRICT_CRC32:
-			checksum = buf_calc_page_crc32(page);
-			break;
-
-		case SRV_CHECKSUM_ALGORITHM_INNODB:
-		case SRV_CHECKSUM_ALGORITHM_STRICT_INNODB:
-			checksum = (ib_uint32_t)
-					buf_calc_page_new_checksum(page);
-			break;
-
-		case SRV_CHECKSUM_ALGORITHM_NONE:
-		case SRV_CHECKSUM_ALGORITHM_STRICT_NONE:
-			checksum = BUF_NO_CHECKSUM_MAGIC;
-			break;
-
-		/* no default so the compiler will emit a warning if new
-		enum is added and not handled here */
-		}
+		checksum = buf_calc_page_crc32(page);
 
 		mach_write_to_4(page + FIL_PAGE_SPACE_OR_CHKSUM, checksum);
 		if (is_log_enabled) {
-			fprintf(log_file, "page::" UINT32PF "; Updated checksum field1"
-				" = " UINT32PF "\n", cur_page_num, checksum);
-		}
-
-		if (write_check == SRV_CHECKSUM_ALGORITHM_STRICT_INNODB
-		    || write_check == SRV_CHECKSUM_ALGORITHM_INNODB) {
-			checksum = (ib_uint32_t)
-					buf_calc_page_old_checksum(page);
+			fprintf(log_file, "page::%llu; Updated checksum"
+				" = %u\n", cur_page_num, checksum);
 		}
 
 		mach_write_to_4(page + physical_page_size -
 				FIL_PAGE_END_LSN_OLD_CHKSUM,checksum);
-
-		if (is_log_enabled) {
-			fprintf(log_file, "page::" UINT32PF "; Updated checksum "
-				"field2 = " UINT32PF "\n", cur_page_num, checksum);
-		}
-
 	}
 
 func_exit:
@@ -793,8 +725,7 @@ write_file(
 	if (physical_page_size
 	    != fwrite(buf, 1, physical_page_size,
 		      file == stdin ? stdout : file)) {
-		fprintf(stderr,
-			"Failed to write page::" UINT32PF " to %s: %s\n",
+		fprintf(stderr, "Failed to write page::%llu to %s: %s\n",
 			cur_page_num, filename, strerror(errno));
 
 		return(false);
@@ -811,16 +742,6 @@ write_file(
 	return(true);
 }
 
-// checks using current xdes page whether the page is free
-static inline bool is_page_free(const byte *xdes, ulint physical_page_size,
-                                uint32_t page_no)
-{
-  const byte *des=
-      xdes + XDES_ARR_OFFSET +
-      XDES_SIZE * ((page_no & (physical_page_size - 1)) / FSP_EXTENT_SIZE);
-  return xdes_is_free(des, page_no % FSP_EXTENT_SIZE);
-}
-
 /*
 Parse the page and collect/dump the information about page type
 @param [in] page	buffer page
@@ -836,10 +757,12 @@ parse_page(
 	bool is_encrypted)
 {
 	unsigned long long id;
-	uint16_t undo_page_type;
+	ulint undo_page_type;
 	char str[20]={'\0'};
 	ulint n_recs;
-	uint32_t page_no, left_page_no, right_page_no;
+	ulint page_no;
+	ulint left_page_no;
+	ulint right_page_no;
 	ulint data_bytes;
 	bool is_leaf;
 	ulint size_range_id;
@@ -854,7 +777,7 @@ parse_page(
 	switch (fil_page_get_type(page)) {
 
 	case FIL_PAGE_INDEX: {
-		uint32_t key_version = mach_read_from_4(page + FIL_PAGE_FILE_FLUSH_LSN_OR_KEY_VERSION);
+		uint key_version = mach_read_from_4(page + FIL_PAGE_FILE_FLUSH_LSN_OR_KEY_VERSION);
 		page_type.n_fil_page_index++;
 
 		/* If page is encrypted we can't read index header */
@@ -877,8 +800,8 @@ parse_page(
 
 			is_leaf = (!*(const uint16*) (page + (PAGE_HEADER + PAGE_LEVEL)));
 
-			if (file) {
-				fprintf(file, "#::" UINT32PF "\t\t|\t\tIndex page\t\t\t|"
+			if (page_type_dump) {
+				fprintf(file, "#::%llu\t\t|\t\tIndex page\t\t\t|"
 					"\tindex id=%llu,", cur_page_num, id);
 
 				fprintf(file,
@@ -896,13 +819,22 @@ parse_page(
 				size_range_id = SIZE_RANGES_FOR_PAGE + 1;
 			}
 			if (per_page_details) {
-				printf("index id=%llu page " UINT32PF " leaf %d n_recs " ULINTPF " data_bytes " ULINTPF
+				printf("index id=%llu page " ULINTPF " leaf %d n_recs " ULINTPF " data_bytes " ULINTPF
 					"\n", id, page_no, is_leaf, n_recs, data_bytes);
 			}
 			/* update per-index statistics */
 			{
-				per_index_stats &index = index_ids[id];
-				if (is_page_free(xdes, physical_page_size, page_no)) {
+				if (index_ids.count(id) == 0) {
+					index_ids[id] = per_index_stats();
+				}
+				std::map<unsigned long long, per_index_stats>::iterator it;
+				it = index_ids.find(id);
+				per_index_stats &index = (it->second);
+				const byte* des = xdes + XDES_ARR_OFFSET
+					+ XDES_SIZE * ((page_no & (physical_page_size - 1))
+						/ FSP_EXTENT_SIZE);
+				if (xdes_is_free(des,
+						 page_no % FSP_EXTENT_SIZE)) {
 					index.free_pages++;
 					return;
 				}
@@ -929,9 +861,9 @@ parse_page(
 				index.total_data_bytes += data_bytes;
 				index.pages_in_size_range[size_range_id] ++;
 			}
-		} else if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tEncrypted Index page\t\t\t|"
-				"\tkey_version " UINT32PF ",%s\n", cur_page_num, key_version, str);
+		} else {
+			fprintf(file, "#::%llu\t\t|\t\tEncrypted Index page\t\t\t|"
+				"\tkey_version %u,%s\n", cur_page_num, key_version, str);
 		}
 
 		break;
@@ -940,17 +872,31 @@ parse_page(
 		page_type.n_fil_page_undo_log++;
 		undo_page_type = mach_read_from_2(page +
 				     TRX_UNDO_PAGE_HDR + TRX_UNDO_PAGE_TYPE);
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tUndo log page\t\t\t|",
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tUndo log page\t\t\t|",
 				cur_page_num);
 		}
-		page_type.n_undo++;
+		if (undo_page_type == TRX_UNDO_INSERT) {
+			page_type.n_undo_insert++;
+			if (page_type_dump) {
+				fprintf(file, "\t%s",
+					"Insert Undo log page");
+			}
+
+		} else if (undo_page_type == TRX_UNDO_UPDATE) {
+			page_type.n_undo_update++;
+			if (page_type_dump) {
+				fprintf(file, "\t%s",
+					"Update undo log page");
+			}
+		}
+
 		undo_page_type = mach_read_from_2(page + TRX_UNDO_SEG_HDR +
 						  TRX_UNDO_STATE);
 		switch (undo_page_type) {
 			case TRX_UNDO_ACTIVE:
 				page_type.n_undo_state_active++;
-				if (file) {
+				if (page_type_dump) {
 					fprintf(file, ", %s", "Undo log of "
 						"an active transaction");
 				}
@@ -958,15 +904,23 @@ parse_page(
 
 			case TRX_UNDO_CACHED:
 				page_type.n_undo_state_cached++;
-				if (file) {
+				if (page_type_dump) {
 					fprintf(file, ", %s", "Page is "
 						"cached for quick reuse");
 				}
 				break;
 
+			case TRX_UNDO_TO_FREE:
+				page_type.n_undo_state_to_free++;
+				if (page_type_dump) {
+					fprintf(file, ", %s", "Insert undo "
+						"segment that can be freed");
+				}
+				break;
+
 			case TRX_UNDO_TO_PURGE:
 				page_type.n_undo_state_to_purge++;
-				if (file) {
+				if (page_type_dump) {
 					fprintf(file, ", %s", "Will be "
 						"freed in purge when all undo"
 					"data in it is removed");
@@ -975,7 +929,7 @@ parse_page(
 
 			case TRX_UNDO_PREPARED:
 				page_type.n_undo_state_prepared++;
-				if (file) {
+				if (page_type_dump) {
 					fprintf(file, ", %s", "Undo log of "
 						"an prepared transaction");
 				}
@@ -985,111 +939,113 @@ parse_page(
 				page_type.n_undo_state_other++;
 				break;
 		}
-		if(file) {
+		if(page_type_dump) {
 			fprintf(file, ", %s\n", str);
 		}
 		break;
 
 	case FIL_PAGE_INODE:
 		page_type.n_fil_page_inode++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tInode page\t\t\t|"
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tInode page\t\t\t|"
 				"\t%s\n",cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_IBUF_FREE_LIST:
 		page_type.n_fil_page_ibuf_free_list++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tInsert buffer free list"
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tInsert buffer free list"
 				" page\t|\t%s\n", cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_TYPE_ALLOCATED:
 		page_type.n_fil_page_type_allocated++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tFreshly allocated "
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tFreshly allocated "
 				"page\t\t|\t%s\n", cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_IBUF_BITMAP:
 		page_type.n_fil_page_ibuf_bitmap++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tInsert Buffer "
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tInsert Buffer "
 				"Bitmap\t\t|\t%s\n", cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_TYPE_SYS:
 		page_type.n_fil_page_type_sys++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tSystem page\t\t\t|"
-				"\t%s\n", cur_page_num, str);
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tSystem page\t\t\t|"
+				"\t%s\n",cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_TYPE_TRX_SYS:
 		page_type.n_fil_page_type_trx_sys++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tTransaction system "
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tTransaction system "
 				"page\t\t|\t%s\n", cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_TYPE_FSP_HDR:
 		page_type.n_fil_page_type_fsp_hdr++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tFile Space "
+		memcpy(xdes, page, physical_page_size);
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tFile Space "
 				"Header\t\t|\t%s\n", cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_TYPE_XDES:
 		page_type.n_fil_page_type_xdes++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tExtent descriptor "
+		memcpy(xdes, page, physical_page_size);
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tExtent descriptor "
 				"page\t\t|\t%s\n", cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_TYPE_BLOB:
 		page_type.n_fil_page_type_blob++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tBLOB page\t\t\t|\t%s\n",
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tBLOB page\t\t\t|\t%s\n",
 				cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_TYPE_ZBLOB:
 		page_type.n_fil_page_type_zblob++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tCompressed BLOB "
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tCompressed BLOB "
 				"page\t\t|\t%s\n", cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_TYPE_ZBLOB2:
 		page_type.n_fil_page_type_zblob2++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tSubsequent Compressed "
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tSubsequent Compressed "
 				"BLOB page\t|\t%s\n", cur_page_num, str);
 		}
 			break;
 
 	case FIL_PAGE_PAGE_COMPRESSED:
 		page_type.n_fil_page_type_page_compressed++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tPage compressed "
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tPage compressed "
 				"page\t|\t%s\n", cur_page_num, str);
 		}
 		break;
 
 	case FIL_PAGE_PAGE_COMPRESSED_ENCRYPTED:
 		page_type.n_fil_page_type_page_compressed_encrypted++;
-		if (file) {
-			fprintf(file, "#::" UINT32PF "\t\t|\t\tPage compressed encrypted "
+		if (page_type_dump) {
+			fprintf(file, "#::%llu\t\t|\t\tPage compressed encrypted "
 				"page\t|\t%s\n", cur_page_num, str);
 		}
 		break;
@@ -1185,11 +1141,15 @@ print_summary(
 
 	fprintf(fil_out, "\n===============================================\n");
 	fprintf(fil_out, "Additional information:\n");
-	fprintf(fil_out, "Undo page type: %d\n", page_type.n_undo);
-	fprintf(fil_out, "Undo page state: %d active, %d cached, %d"
+	fprintf(fil_out, "Undo page type: %d insert, %d update, %d other\n",
+		page_type.n_undo_insert,
+		page_type.n_undo_update,
+		page_type.n_undo_other);
+	fprintf(fil_out, "Undo page state: %d active, %d cached, %d to_free, %d"
 		" to_purge, %d prepared, %d other\n",
 		page_type.n_undo_state_active,
 		page_type.n_undo_state_cached,
+		page_type.n_undo_state_to_free,
 		page_type.n_undo_state_to_purge,
 		page_type.n_undo_state_prepared,
 		page_type.n_undo_state_other);
@@ -1241,25 +1201,19 @@ static struct my_option innochecksum_options[] = {
   {"count", 'c', "Print the count of pages in the file and exits.",
     &just_count, &just_count, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"start_page", 's', "Start on this page number (0 based).",
-    &start_page, &start_page, 0, GET_UINT, REQUIRED_ARG,
-    0, 0, FIL_NULL, 0, 1, 0},
+    &start_page, &start_page, 0, GET_ULL, REQUIRED_ARG,
+    0, 0, ULLONG_MAX, 0, 1, 0},
   {"end_page", 'e', "End at this page number (0 based).",
-    &end_page, &end_page, 0, GET_UINT, REQUIRED_ARG,
-    0, 0, FIL_NULL, 0, 1, 0},
+    &end_page, &end_page, 0, GET_ULL, REQUIRED_ARG,
+    0, 0, ULLONG_MAX, 0, 1, 0},
   {"page", 'p', "Check only this page (0 based).",
-    &do_page, &do_page, 0, GET_UINT, REQUIRED_ARG,
-    0, 0, FIL_NULL, 0, 1, 0},
-  {"strict-check", 'C', "Specify the strict checksum algorithm by the user.",
-    &strict_check, &strict_check, &innochecksum_algorithms_typelib,
-    GET_ENUM, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+    &do_page, &do_page, 0, GET_ULL, REQUIRED_ARG,
+    0, 0, ULLONG_MAX, 0, 1, 0},
   {"no-check", 'n', "Ignore the checksum verification.",
     &no_check, &no_check, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"allow-mismatches", 'a', "Maximum checksum mismatch allowed.",
     &allow_mismatches, &allow_mismatches, 0,
     GET_ULL, REQUIRED_ARG, 0, 0, ULLONG_MAX, 0, 1, 0},
-  {"write", 'w', "Rewrite the checksum algorithm by the user.",
-    &write_check, &write_check, &innochecksum_algorithms_typelib,
-    GET_ENUM, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"page-type-summary", 'S', "Display a count of each page type "
    "in a tablespace.", &page_type_summary, &page_type_summary, 0,
    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
@@ -1300,7 +1254,7 @@ static void usage(void)
 	printf("InnoDB offline file checksum utility.\n");
 	printf("Usage: %s [-c] [-s <start page>] [-e <end page>] "
 		"[-p <page>] [-i] [-v]  [-a <allow mismatches>] [-n] "
-		"[-C <strict-check>] [-w <write>] [-S] [-D <page type dump>] "
+		"[-S] [-D <page type dump>] "
 		"[-l <log>] [-l] [-m <merge pages>] <filename or [-]>\n", my_progname);
 	printf("See https://mariadb.com/kb/en/library/innochecksum/"
 	       " for usage hints.\n");
@@ -1337,38 +1291,6 @@ innochecksum_get_one_option(
 		my_end(0);
 		exit(EXIT_SUCCESS);
 		break;
-	case 'C':
-		strict_verify = true;
-		switch ((srv_checksum_algorithm_t) strict_check) {
-
-		case SRV_CHECKSUM_ALGORITHM_STRICT_CRC32:
-		case SRV_CHECKSUM_ALGORITHM_CRC32:
-			srv_checksum_algorithm =
-				SRV_CHECKSUM_ALGORITHM_STRICT_CRC32;
-			break;
-
-		case SRV_CHECKSUM_ALGORITHM_STRICT_INNODB:
-		case SRV_CHECKSUM_ALGORITHM_INNODB:
-			srv_checksum_algorithm =
-				SRV_CHECKSUM_ALGORITHM_STRICT_INNODB;
-			break;
-
-		case SRV_CHECKSUM_ALGORITHM_STRICT_NONE:
-		case SRV_CHECKSUM_ALGORITHM_NONE:
-			srv_checksum_algorithm =
-				SRV_CHECKSUM_ALGORITHM_STRICT_NONE;
-			break;
-
-		case SRV_CHECKSUM_ALGORITHM_STRICT_FULL_CRC32:
-		case SRV_CHECKSUM_ALGORITHM_FULL_CRC32:
-			srv_checksum_algorithm =
-				SRV_CHECKSUM_ALGORITHM_STRICT_FULL_CRC32;
-			break;
-
-		default:
-			return(true);
-		}
-		break;
 	case 'n':
 		no_check = true;
 		break;
@@ -1444,14 +1366,14 @@ static bool check_encryption(const char* filename, const byte* page)
 		return false;
 	}
 
-	uint32_t min_key_version = mach_read_from_4
+	uint min_key_version = mach_read_from_4
 		(page + offset + MAGIC_SZ + 2 + iv_length);
 
-	uint32_t key_id = mach_read_from_4
+	uint key_id = mach_read_from_4
 		(page + offset + MAGIC_SZ + 2 + iv_length + 4);
 
 	if (type == CRYPT_SCHEME_1 && is_log_enabled) {
-		fprintf(log_file,"Tablespace %s encrypted key_version " UINT32PF " key_id " UINT32PF "\n",
+		fprintf(log_file,"Tablespace %s encrypted key_version %u key_id %u\n",
 			filename, min_key_version, key_id);
 	}
 
@@ -1473,7 +1395,7 @@ static int verify_checksum(
 {
 	int exit_status = 0;
 	if (is_page_corrupted(buf, is_encrypted, flags)) {
-		fprintf(stderr, "Fail: page::" UINT32PF " invalid\n",
+		fprintf(stderr, "Fail: page::%llu invalid\n",
 			cur_page_num);
 
 		(*mismatch_count)++;
@@ -1547,7 +1469,7 @@ int main(
 	/* size of file (has to be 64 bits) */
 	unsigned long long int	size		= 0;
 	/* number of pages in file */
-	uint32_t	pages;
+	ulint		pages;
 
 	off_t		offset			= 0;
 	/* count the no. of page corrupted. */
@@ -1571,13 +1493,6 @@ int main(
 		goto my_exit;
 	}
 
-	if (strict_verify && no_check) {
-		fprintf(stderr, "Error: --strict-check option cannot be used "
-			"together with --no-check option.\n");
-		exit_status = 1;
-		goto my_exit;
-	}
-
 	if (no_check && !do_write) {
 		fprintf(stderr, "Error: --no-check must be associated with "
 			"--write option.\n");
@@ -1747,38 +1662,36 @@ int main(
 		}
 
 		if (per_page_details) {
-			printf("page " UINT32PF " ", cur_page_num);
+			printf("page %llu ", cur_page_num);
 		}
 
-		memcpy(xdes, buf, physical_page_size);
-
 		if (page_type_summary || page_type_dump) {
 			parse_page(buf, xdes, fil_page_type, is_encrypted);
 		}
 
-		pages = uint32_t(size / physical_page_size);
+		pages = (ulint) (size / physical_page_size);
 
 		if (just_count) {
-			fprintf(read_from_stdin ? stderr : stdout,
-				"Number of pages:" UINT32PF "\n", pages);
+			if (read_from_stdin) {
+				fprintf(stderr, "Number of pages:" ULINTPF "\n", pages);
+			} else {
+				printf("Number of pages:" ULINTPF "\n", pages);
+			}
 			continue;
 		} else if (verbose && !read_from_stdin) {
 			if (is_log_enabled) {
 				fprintf(log_file, "file %s = %llu bytes "
-					"(" UINT32PF " pages)\n",
-					filename, size, pages);
+					"(" ULINTPF " pages)\n", filename, size, pages);
 				if (do_one_page) {
 					fprintf(log_file, "Innochecksum: "
-						"checking page::"
-						UINT32PF ";\n",
+						"checking page::%llu;\n",
 						do_page);
 				}
 			}
 		} else {
 			if (is_log_enabled) {
 				fprintf(log_file, "Innochecksum: checking "
-					"pages in range::" UINT32PF
-					" to " UINT32PF "\n",
+					"pages in range::%llu to %llu\n",
 					start_page, use_end_page ?
 					end_page : (pages - 1));
 			}
@@ -1822,8 +1735,8 @@ int main(
 				the desired page. */
 				partial_page_read = false;
 
-				offset = off_t(ulonglong(start_page)
-					       * physical_page_size);
+				offset = (off_t) start_page
+					* (off_t) physical_page_size;
 				if (IF_WIN(_fseeki64,fseeko)(fil_in, offset,
 							     SEEK_SET)) {
 					perror("Error: Unable to seek to "
@@ -1864,7 +1777,12 @@ int main(
 					count++;
 
 					if (!bytes || feof(fil_in)) {
-						goto unexpected_eof;
+						fprintf(stderr, "Error: Unable "
+							"to seek to necessary "
+							"offset");
+
+						exit_status = 1;
+						goto my_exit;
 					}
 				}
 			}
@@ -1880,31 +1798,10 @@ int main(
 			partial_page_read = false;
 
 			if (!bytes && feof(fil_in)) {
-				if (cur_page_num == start_page) {
-unexpected_eof:
-					fputs("Error: Unable "
-					      "to seek to necessary offset\n",
-					      stderr);
-
-					exit_status = 1;
-					goto my_exit;
-				}
 				break;
 			}
 
 			if (ferror(fil_in)) {
-#ifdef _AIX
-				/*
-				  AIX fseeko can go past eof without error.
-				  the error occurs on read, hence output the
-				  same error here as would show up on other
-				  platforms. This shows up in the mtr test
-				  innodb_zip.innochecksum_3-4k,crc32,innodb
-				*/
-				if (errno == EFBIG) {
-					goto unexpected_eof;
-				}
-#endif
 				fprintf(stderr, "Error reading " ULINTPF " bytes",
 					physical_page_size);
 				perror(" ");
@@ -1942,7 +1839,6 @@ first_non_zero:
 			checksum verification.*/
 			if (!no_check
 			    && !skip_page
-			    && !is_page_free(xdes, physical_page_size, cur_page_num)
 			    && (exit_status = verify_checksum(
 						buf, is_encrypted,
 						&mismatch_count, flags))) {
@@ -1961,11 +1857,7 @@ first_non_zero:
 			}
 
 			if (per_page_details) {
-				printf("page " UINT32PF " ", cur_page_num);
-			}
-
-			if (page_get_page_no(buf) % physical_page_size == 0) {
-				memcpy(xdes, buf, physical_page_size);
+				printf("page %llu ", cur_page_num);
 			}
 
 			if (page_type_summary || page_type_dump) {
@@ -1981,10 +1873,10 @@ first_non_zero:
 					if (!lastt) {
 						lastt= now;
 					} else if (now - lastt >= 1 && is_log_enabled) {
-						fprintf(log_file, "page::" UINT32PF " "
+						fprintf(log_file, "page::%llu "
 							"okay: %.3f%% done\n",
 							(cur_page_num - 1),
-							(double) cur_page_num / pages * 100);
+							(float) cur_page_num / pages * 100);
 						lastt = now;
 					}
 				}
