@@ -16,7 +16,7 @@
 
 
 #define PLUGIN_VERSION 0x104
-#define PLUGIN_STR_VERSION "1.4.10"
+#define PLUGIN_STR_VERSION "1.4.13"
 
 #define _my_thread_var loc_thread_var
 
@@ -124,7 +124,7 @@ static char *default_home= (char *)".";
 #define my_rename(A, B, C) loc_rename(A, B)
 #define my_tell(A, B) loc_tell(A)
 #define my_write(A, B, C, D) loc_write(A, B, C)
-#define my_malloc(A, B) malloc(A)
+#define my_malloc(A, B, C) malloc(B)
 #define my_free(A) free(A)
 #ifdef my_errno
   #undef my_errno
@@ -140,6 +140,14 @@ static int loc_file_errno;
 #define logger_write loc_logger_write
 #define logger_rotate loc_logger_rotate
 #define logger_init_mutexts loc_logger_init_mutexts
+#define logger_time_to_rotate loc_logger_time_to_rotate
+
+#ifndef HOSTNAME_LENGTH
+#define HOSTNAME_LENGTH 255
+#endif
+#ifndef USERNAME_CHAR_LENGTH
+#define USERNAME_CHAR_LENGTH 128
+#endif
 
 
 static size_t loc_write(File Filedes, const uchar *Buffer, size_t Count)
@@ -276,8 +284,9 @@ static my_off_t loc_tell(File fd)
 #endif /*WIN32*/
 
 
-extern char server_version[];
+extern MYSQL_PLUGIN_IMPORT char server_version[];
 static const char *serv_ver= NULL;
+const char *(*thd_priv_host_ptr)(MYSQL_THD thd, size_t *length);
 static int started_mysql= 0;
 static int mysql_57_started= 0;
 static int debug_server_started= 0;
@@ -304,7 +313,7 @@ static char *big_buffer= NULL;
 static size_t big_buffer_alloced= 0;
 static unsigned int query_log_limit= 0;
 
-static char servhost[256];
+static char servhost[HOSTNAME_LENGTH+1];
 static uint servhost_len;
 static char *syslog_ident;
 static char syslog_ident_buffer[128]= "mysql-server_auditing";
@@ -316,9 +325,9 @@ struct connection_info
   unsigned long long query_id;
   char db[256];
   int db_length;
-  char user[64];
+  char user[USERNAME_CHAR_LENGTH+1];
   int user_length;
-  char host[64];
+  char host[HOSTNAME_LENGTH+1];
   int host_length;
   char ip[64];
   int ip_length;
@@ -327,9 +336,9 @@ struct connection_info
   char query_buffer[1024];
   time_t query_time;
   int log_always;
-  char proxy[64];
+  char proxy[USERNAME_CHAR_LENGTH+1];
   int proxy_length;
-  char proxy_host[64];
+  char proxy_host[HOSTNAME_LENGTH+1];
   int proxy_host_length;
 };
 
@@ -554,22 +563,22 @@ static struct st_mysql_show_var audit_status[]=
   {0,0,0}
 };
 
-#if defined(HAVE_PSI_INTERFACE) && !defined(FLOGGER_NO_PSI)
-/* These belong to the service initialization */
+#ifdef HAVE_PSI_INTERFACE
 static PSI_mutex_key key_LOCK_operations;
-static PSI_mutex_key key_LOCK_atomic;
-static PSI_mutex_key key_LOCK_bigbuffer;
 static PSI_mutex_info mutex_key_list[]=
 {
   { &key_LOCK_operations, "SERVER_AUDIT_plugin::lock_operations",
-    PSI_FLAG_GLOBAL},
+    PSI_FLAG_GLOBAL}
+#ifndef FLOGGER_NO_PSI
+  ,
   { &key_LOCK_atomic, "SERVER_AUDIT_plugin::lock_atomic",
     PSI_FLAG_GLOBAL},
   { &key_LOCK_bigbuffer, "SERVER_AUDIT_plugin::lock_bigbuffer",
     PSI_FLAG_GLOBAL}
+#endif /*FLOGGER_NO_PSI*/
 };
-#endif
-static mysql_mutex_t lock_operations;
+#endif /*HAVE_PSI_INTERFACE*/
+static mysql_prlock_t lock_operations;
 static mysql_mutex_t lock_atomic;
 static mysql_mutex_t lock_bigbuffer;
 
@@ -709,6 +718,8 @@ static char *coll_search(struct user_coll *c, const char *n, size_t len)
 {
   struct user_name un;
   struct user_name *found;
+  if (!c->n_users)
+    return 0;
   un.name_len= len;
   un.name= (char *) n;
   found= (struct user_name*)  bsearch(&un, c->users, c->n_users,
@@ -739,7 +750,8 @@ static int coll_insert(struct user_coll *c, char *n, size_t len)
 
 static void coll_sort(struct user_coll *c)
 {
-  qsort(c->users, c->n_users, sizeof(c->users[0]), cmp_users);
+  if (c->n_users)
+    qsort(c->users, c->n_users, sizeof(c->users[0]), cmp_users);
 }
 
 
@@ -816,6 +828,7 @@ enum sa_keywords
   SQLCOM_DML,
   SQLCOM_GRANT,
   SQLCOM_CREATE_USER,
+  SQLCOM_ALTER_USER,
   SQLCOM_CHANGE_MASTER,
   SQLCOM_CREATE_SERVER,
   SQLCOM_SET_OPTION,
@@ -854,12 +867,8 @@ struct sa_keyword keywords_to_skip[]=
 
 struct sa_keyword not_ddl_keywords[]=
 {
-  {4, "DROP", &function_word, SQLCOM_QUERY_ADMIN},
-  {4, "DROP", &procedure_word, SQLCOM_QUERY_ADMIN},
   {4, "DROP", &user_word, SQLCOM_DCL},
   {6, "CREATE", &user_word, SQLCOM_DCL},
-  {6, "CREATE", &function_word, SQLCOM_QUERY_ADMIN},
-  {6, "CREATE", &procedure_word, SQLCOM_QUERY_ADMIN},
   {6, "RENAME", &user_word, SQLCOM_DCL},
   {0, NULL, 0, SQLCOM_DDL}
 };
@@ -923,6 +932,7 @@ struct sa_keyword passwd_keywords[]=
 {
   {3, "SET", &password_word, SQLCOM_SET_OPTION},
   {5, "ALTER", &server_word, SQLCOM_ALTER_SERVER},
+  {5, "ALTER", &user_word, SQLCOM_ALTER_USER},
   {5, "GRANT", 0, SQLCOM_GRANT},
   {6, "CREATE", &user_word, SQLCOM_CREATE_USER},
   {6, "CREATE", &server_word, SQLCOM_CREATE_SERVER},
@@ -970,7 +980,8 @@ static void get_str_n(char *dest, int *dest_len, size_t dest_size,
   if (src_len >= dest_size)
     src_len= dest_size - 1;
 
-  memcpy(dest, src, src_len);
+  if (src_len)
+    memcpy(dest, src, src_len);
   dest[src_len]= 0;
   *dest_len= (int)src_len;
 }
@@ -1139,10 +1150,10 @@ static void setup_connection_simple(struct connection_info *ci)
 }
 
 
-#define MAX_HOSTNAME 61
+#define MAX_HOSTNAME  (HOSTNAME_LENGTH + 1)	/* len+1 in mysql.user */
 #define USERNAME_LENGTH 384
 
-static void setup_connection_connect(struct connection_info *cn,
+static void setup_connection_connect(MYSQL_THD thd,struct connection_info *cn,
     const struct mysql_event_connection *event)
 {
   cn->query_id= 0;
@@ -1160,17 +1171,26 @@ static void setup_connection_connect(struct connection_info *cn,
   cn->header= 0;
   if (event->proxy_user && event->proxy_user[0])
   {
-    const char *priv_host= event->proxy_user +
-            sizeof(char[MAX_HOSTNAME+USERNAME_LENGTH+5]);
+    const char *priv_host;
     size_t priv_host_length;
 
-    if (mysql_57_started)
+    if (thd_priv_host_ptr)
     {
-      priv_host+= sizeof(size_t);
-      priv_host_length= *(size_t *) (priv_host + MAX_HOSTNAME);
+      priv_host= (*thd_priv_host_ptr)(thd, &priv_host_length);
     }
     else
-      priv_host_length= strlen(priv_host);
+    {
+      // 5 is "'" around host and user and "@"
+      priv_host= event->proxy_user +
+            sizeof(char[MAX_HOSTNAME + USERNAME_LENGTH + 5]);
+      if (mysql_57_started)
+      {
+        priv_host+= sizeof(size_t);
+        priv_host_length= *(size_t *) (priv_host + MAX_HOSTNAME);
+      }
+      else
+        priv_host_length= strlen(priv_host);
+    }
 
 
     get_str_n(cn->proxy, &cn->proxy_length, sizeof(cn->proxy),
@@ -1316,19 +1336,41 @@ static void change_connection(struct connection_info *cn,
             event->ip, event->ip_length);
 }
 
+/*
+  Write to the log
+
+  @param take_lock  If set, take a read lock (or write lock on rotate).
+                    If not set, the caller has a already taken a write lock
+*/
+
 static int write_log(const char *message, size_t len, int take_lock)
 {
   int result= 0;
   if (take_lock)
-    flogger_mutex_lock(&lock_operations);
+  {
+    /* Start by taking a read lock */
+    mysql_prlock_rdlock(&lock_operations);
+  }
 
   if (output_type == OUTPUT_FILE)
   {
-    if (logfile &&
-        (is_active= (logger_write(logfile, message, len) == (int) len)))
-      goto exit;
-    ++log_write_failures;
-    result= 1;
+    if (logfile)
+    {
+      my_bool allow_rotate= !take_lock; /* Allow rotate if caller write lock */
+      if (take_lock && logger_time_to_rotate(logfile))
+      {
+        /* We have to rotate the log, change above read lock to write lock */
+        mysql_prlock_unlock(&lock_operations);
+        mysql_prlock_wrlock(&lock_operations);
+        allow_rotate= 1;
+      }
+      if (!(is_active= (logger_write_r(logfile, allow_rotate, message, len) ==
+                        (int) len)))
+      {
+        ++log_write_failures;
+        result= 1;
+      }
+    }
   }
   else if (output_type == OUTPUT_SYSLOG)
   {
@@ -1336,9 +1378,8 @@ static int write_log(const char *message, size_t len, int take_lock)
            syslog_priority_codes[syslog_priority],
            "%s %.*s", syslog_info, (int) len, message);
   }
-exit:
   if (take_lock)
-    flogger_mutex_unlock(&lock_operations);
+    mysql_prlock_unlock(&lock_operations);
   return result;
 }
 
@@ -1577,27 +1618,32 @@ no_password:
 
 
 
-static int do_log_user(const char *name, int take_lock)
+static int do_log_user(const char *name, int len,
+                       const char *proxy, int proxy_len, int take_lock)
 {
-  size_t len;
   int result;
 
   if (!name)
     return 0;
-  len= strlen(name);
 
   if (take_lock)
-    flogger_mutex_lock(&lock_operations);
+    mysql_prlock_rdlock(&lock_operations);
 
   if (incl_user_coll.n_users)
-    result= coll_search(&incl_user_coll, name, len) != 0;
+  {
+    result= coll_search(&incl_user_coll, name, len) != 0 ||
+            (proxy && coll_search(&incl_user_coll, proxy, proxy_len) != 0);
+  }
   else if (excl_user_coll.n_users)
-    result=  coll_search(&excl_user_coll, name, len) == 0;
+  {
+    result= coll_search(&excl_user_coll, name, len) == 0 &&
+            (proxy && coll_search(&excl_user_coll, proxy, proxy_len) == 0);
+  }
   else
     result= 1;
 
   if (take_lock)
-    flogger_mutex_unlock(&lock_operations);
+    mysql_prlock_unlock(&lock_operations);
   return result;
 }
 
@@ -1815,6 +1861,7 @@ do_log_query:
   {
     case SQLCOM_GRANT:
     case SQLCOM_CREATE_USER:
+    case SQLCOM_ALTER_USER:
       csize+= escape_string_hide_passwords(query, query_len,
                                            uh_buffer, uh_buffer_size,
                                            "IDENTIFIED", 10, "BY", 2, 0);
@@ -1944,7 +1991,7 @@ static struct connection_info ci_disconnect_buffer;
 #define AA_FREE_CONNECTION 1
 #define AA_CHANGE_USER 2
 
-static void update_connection_info(struct connection_info *cn,
+static void update_connection_info(MYSQL_THD thd, struct connection_info *cn,
     unsigned int event_class, const void *ev, int *after_action)
 {
   *after_action= 0;
@@ -2065,14 +2112,10 @@ static void update_connection_info(struct connection_info *cn,
     switch (event->event_subclass)
     {
       case MYSQL_AUDIT_CONNECTION_CONNECT:
-        setup_connection_connect(cn, event);
-        if (event->status == 0 && event->proxy_user && event->proxy_user[0])
-          log_proxy(cn, event);
+        setup_connection_connect(thd, cn, event);
         break;
       case MYSQL_AUDIT_CONNECTION_CHANGE_USER:
         *after_action= AA_CHANGE_USER;
-        if (event->proxy_user && event->proxy_user[0])
-          log_proxy(cn, event);
         break;
       default:;
     }
@@ -2127,7 +2170,7 @@ void auditing(MYSQL_THD thd, unsigned int event_class, const void *ev)
     cn= get_loc_info(thd);
   }
 
-  update_connection_info(cn, event_class, ev, &after_action);
+  update_connection_info(thd, cn, event_class, ev, &after_action);
 
   if (!logging)
   {
@@ -2137,7 +2180,9 @@ void auditing(MYSQL_THD thd, unsigned int event_class, const void *ev)
   }
 
   if (event_class == MYSQL_AUDIT_GENERAL_CLASS && FILTER(EVENT_QUERY) &&
-      cn && (cn->log_always || do_log_user(cn->user, 1)))
+      cn && (cn->log_always || do_log_user(cn->user, cn->user_length,
+                                           cn->proxy, cn->proxy_length,
+                                           1)))
   {
     const struct mysql_event_general *event =
       (const struct mysql_event_general *) ev;
@@ -2157,7 +2202,8 @@ void auditing(MYSQL_THD thd, unsigned int event_class, const void *ev)
   {
     const struct mysql_event_table *event =
       (const struct mysql_event_table *) ev;
-    if (do_log_user(event->user, 1))
+    if (do_log_user(event->user, (int) SAFE_STRLEN(event->user),
+                    cn->proxy, cn->proxy_length, 1))
     {
       switch (event->event_subclass)
       {
@@ -2190,6 +2236,8 @@ void auditing(MYSQL_THD thd, unsigned int event_class, const void *ev)
     {
       case MYSQL_AUDIT_CONNECTION_CONNECT:
         log_connection(cn, event, event->status ? "FAILED_CONNECT": "CONNECT");
+        if (event->status == 0 && event->proxy_user && event->proxy_user[0])
+          log_proxy(cn, event);
         break;
       case MYSQL_AUDIT_CONNECTION_DISCONNECT:
         if (use_event_data_for_disconnect)
@@ -2199,6 +2247,8 @@ void auditing(MYSQL_THD thd, unsigned int event_class, const void *ev)
         break;
       case MYSQL_AUDIT_CONNECTION_CHANGE_USER:
         log_connection(cn, event, "CHANGEUSER");
+        if (event->proxy_user && event->proxy_user[0])
+          log_proxy(cn, event);
         break;
       default:;
     }
@@ -2399,7 +2449,6 @@ typedef struct loc_system_variables
   ulong max_tmp_tables;
   ulong max_insert_delayed_threads;
   ulong min_examined_row_limit;
-  ulong multi_range_count;
   ulong net_buffer_length;
   ulong net_interactive_timeout;
   ulong net_read_timeout;
@@ -2439,31 +2488,39 @@ typedef struct loc_system_variables
 
 static int init_done= 0;
 
+static void* find_sym(const char *sym)
+{
+#ifdef _WIN32
+  return GetProcAddress(GetModuleHandle("server.dll"),sym);
+#else
+  return dlsym(RTLD_DEFAULT, sym);
+#endif
+}
+
 static int server_audit_init(void *p __attribute__((unused)))
 {
   if (!serv_ver)
   {
-#ifdef _WIN32
-    serv_ver= (const char *) GetProcAddress(0, "server_version");
-#else
-    serv_ver= server_version;
-#endif /*_WIN32*/
+    serv_ver= find_sym("server_version");
   }
+
   if (!mysql_57_started)
   {
-    const void *my_hash_init_ptr= dlsym(RTLD_DEFAULT, "_my_hash_init");
+    const void *my_hash_init_ptr= find_sym("_my_hash_init");
     if (!my_hash_init_ptr)
     {
       maria_above_5= 1;
-      my_hash_init_ptr= dlsym(RTLD_DEFAULT, "my_hash_init2");
+      my_hash_init_ptr= find_sym("my_hash_init2");
     }
     if (!my_hash_init_ptr)
       return 1;
+
+    thd_priv_host_ptr= dlsym(RTLD_DEFAULT, "thd_priv_host");
   }
 
-  if(!(int_mysql_data_home= dlsym(RTLD_DEFAULT, "mysql_data_home")))
+  if(!(int_mysql_data_home= find_sym("mysql_data_home")))
   {
-    if(!(int_mysql_data_home= dlsym(RTLD_DEFAULT, "?mysql_data_home@@3PADA")))
+    if(!(int_mysql_data_home= find_sym("?mysql_data_home@@3PADA")))
       int_mysql_data_home= &default_home;
   }
 
@@ -2485,11 +2542,11 @@ static int server_audit_init(void *p __attribute__((unused)))
   servhost_len= (uint)strlen(servhost);
 
   logger_init_mutexes();
-#if defined(HAVE_PSI_INTERFACE) && !defined(FLOGGER_NO_PSI)
+#ifdef HAVE_PSI_INTERFACE
   if (PSI_server)
     PSI_server->register_mutex("server_audit", mutex_key_list, 1);
 #endif
-  flogger_mutex_init(key_LOCK_operations, &lock_operations, MY_MUTEX_INIT_FAST);
+  mysql_prlock_init(key_LOCK_operations, &lock_operations);
   flogger_mutex_init(key_LOCK_operations, &lock_atomic, MY_MUTEX_INIT_FAST);
   flogger_mutex_init(key_LOCK_operations, &lock_bigbuffer, MY_MUTEX_INIT_FAST);
 
@@ -2577,7 +2634,7 @@ static int server_audit_deinit(void *p __attribute__((unused)))
     closelog();
 
   (void) free(big_buffer);
-  flogger_mutex_destroy(&lock_operations);
+  mysql_prlock_destroy(&lock_operations);
   flogger_mutex_destroy(&lock_atomic);
   flogger_mutex_destroy(&lock_bigbuffer);
 
@@ -2688,7 +2745,7 @@ static void update_file_path(MYSQL_THD thd,
   fprintf(stderr, "Log file name was changed to '%s'.\n", new_name);
 
   if (!maria_55_started || !debug_server_started)
-    flogger_mutex_lock(&lock_operations);
+    mysql_prlock_wrlock(&lock_operations);
 
   if (logging)
     log_current_query(thd);
@@ -2720,7 +2777,7 @@ static void update_file_path(MYSQL_THD thd,
   file_path= path_buffer;
 exit_func:
   if (!maria_55_started || !debug_server_started)
-    flogger_mutex_unlock(&lock_operations);
+    mysql_prlock_unlock(&lock_operations);
   ADD_ATOMIC(internal_stop_logging, -1);
 }
 
@@ -2736,9 +2793,9 @@ static void update_file_rotations(MYSQL_THD thd  __attribute__((unused)),
   if (!logging || output_type != OUTPUT_FILE)
     return;
 
-  flogger_mutex_lock(&lock_operations);
+  mysql_prlock_wrlock(&lock_operations);
   logfile->rotations= rotations;
-  flogger_mutex_unlock(&lock_operations);
+  mysql_prlock_unlock(&lock_operations);
 }
 
 
@@ -2754,9 +2811,9 @@ static void update_file_rotate_size(MYSQL_THD thd  __attribute__((unused)),
   if (!logging || output_type != OUTPUT_FILE)
     return;
 
-  flogger_mutex_lock(&lock_operations);
+  mysql_prlock_wrlock(&lock_operations);
   logfile->size_limit= file_rotate_size;
-  flogger_mutex_unlock(&lock_operations);
+  mysql_prlock_unlock(&lock_operations);
 }
 
 
@@ -2801,7 +2858,7 @@ static void update_incl_users(MYSQL_THD thd,
   char *new_users= (*(char **) save) ? *(char **) save : empty_str;
   size_t new_len= strlen(new_users) + 1;
   if (!maria_55_started || !debug_server_started)
-    flogger_mutex_lock(&lock_operations);
+    mysql_prlock_wrlock(&lock_operations);
   mark_always_logged(thd);
 
   if (new_len > sizeof(incl_user_buffer))
@@ -2815,7 +2872,7 @@ static void update_incl_users(MYSQL_THD thd,
   error_header();
   fprintf(stderr, "server_audit_incl_users set to '%s'.\n", incl_users);
   if (!maria_55_started || !debug_server_started)
-    flogger_mutex_unlock(&lock_operations);
+    mysql_prlock_unlock(&lock_operations);
 }
 
 
@@ -2826,7 +2883,7 @@ static void update_excl_users(MYSQL_THD thd  __attribute__((unused)),
   char *new_users= (*(char **) save) ? *(char **) save : empty_str;
   size_t new_len= strlen(new_users) + 1;
   if (!maria_55_started || !debug_server_started)
-    flogger_mutex_lock(&lock_operations);
+    mysql_prlock_wrlock(&lock_operations);
   mark_always_logged(thd);
 
   if (new_len > sizeof(excl_user_buffer))
@@ -2840,7 +2897,7 @@ static void update_excl_users(MYSQL_THD thd  __attribute__((unused)),
   error_header();
   fprintf(stderr, "server_audit_excl_users set to '%s'.\n", excl_users);
   if (!maria_55_started || !debug_server_started)
-    flogger_mutex_unlock(&lock_operations);
+    mysql_prlock_unlock(&lock_operations);
 }
 
 
@@ -2853,7 +2910,7 @@ static void update_output_type(MYSQL_THD thd,
     return;
 
   ADD_ATOMIC(internal_stop_logging, 1);
-  flogger_mutex_lock(&lock_operations);
+  mysql_prlock_wrlock(&lock_operations);
   if (logging)
   {
     log_current_query(thd);
@@ -2867,7 +2924,7 @@ static void update_output_type(MYSQL_THD thd,
 
   if (logging)
     start_logging();
-  flogger_mutex_unlock(&lock_operations);
+  mysql_prlock_unlock(&lock_operations);
   ADD_ATOMIC(internal_stop_logging, -1);
 }
 
@@ -2897,9 +2954,9 @@ static void update_syslog_priority(MYSQL_THD thd  __attribute__((unused)),
   if (syslog_priority == new_priority)
     return;
 
-  flogger_mutex_lock(&lock_operations);
+  mysql_prlock_wrlock(&lock_operations);
   mark_always_logged(thd);
-  flogger_mutex_unlock(&lock_operations);
+  mysql_prlock_unlock(&lock_operations);
   error_header();
   fprintf(stderr, "SysLog priority was changed from '%s' to '%s'.\n",
           syslog_priority_names[syslog_priority],
@@ -2918,7 +2975,7 @@ static void update_logging(MYSQL_THD thd,
 
   ADD_ATOMIC(internal_stop_logging, 1);
   if (!maria_55_started || !debug_server_started)
-    flogger_mutex_lock(&lock_operations);
+    mysql_prlock_wrlock(&lock_operations);
   if ((logging= new_logging))
   {
     start_logging();
@@ -2935,7 +2992,7 @@ static void update_logging(MYSQL_THD thd,
   }
 
   if (!maria_55_started || !debug_server_started)
-    flogger_mutex_unlock(&lock_operations);
+    mysql_prlock_unlock(&lock_operations);
   ADD_ATOMIC(internal_stop_logging, -1);
 }
 
@@ -2950,13 +3007,13 @@ static void update_mode(MYSQL_THD thd  __attribute__((unused)),
 
   ADD_ATOMIC(internal_stop_logging, 1);
   if (!maria_55_started || !debug_server_started)
-    flogger_mutex_lock(&lock_operations);
+    mysql_prlock_wrlock(&lock_operations);
   mark_always_logged(thd);
   error_header();
   fprintf(stderr, "Logging mode was changed from %d to %d.\n", mode, new_mode);
   mode= new_mode;
   if (!maria_55_started || !debug_server_started)
-    flogger_mutex_unlock(&lock_operations);
+    mysql_prlock_unlock(&lock_operations);
   ADD_ATOMIC(internal_stop_logging, -1);
 }
 
@@ -2971,14 +3028,14 @@ static void update_syslog_ident(MYSQL_THD thd  __attribute__((unused)),
   syslog_ident= syslog_ident_buffer;
   error_header();
   fprintf(stderr, "SYSYLOG ident was changed to '%s'\n", syslog_ident);
-  flogger_mutex_lock(&lock_operations);
+  mysql_prlock_wrlock(&lock_operations);
   mark_always_logged(thd);
   if (logging && output_type == OUTPUT_SYSLOG)
   {
     stop_logging();
     start_logging();
   }
-  flogger_mutex_unlock(&lock_operations);
+  mysql_prlock_unlock(&lock_operations);
 }
 
 
@@ -2995,7 +3052,7 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
   if (fdwReason != DLL_PROCESS_ATTACH)
     return 1;
 
-  serv_ver= (const char *) GetProcAddress(0, "server_version");
+  serv_ver= server_version;
 #else
 void __attribute__ ((constructor)) audit_plugin_so_init(void)
 {
