@@ -31,7 +31,6 @@ defaults=""
 defaults_group_suffix=""
 mysqld_opt=""
 user=""
-group=""
 silent_startup="--silent-startup"
 
 force=0
@@ -96,11 +95,6 @@ Usage: $0 [OPTIONS]
                        user.  You must be root to use this option.  By default
                        mysqld runs using your current login name and files and
                        directories that it creates will be owned by you.
-  --group=group_name   The login group to use for running mysqld.  Files and
-                       directories created by mysqld will be owned by this
-                       group. You must be root to use this option.  By default
-                       mysqld runs using your current group and files and
-                       directories that it creates will be owned by you.
 
 All other options are passed to the mysqld program
 
@@ -150,11 +144,11 @@ parse_arguments()
       --ldata=*|--datadir=*|--data=*) ldata=`parse_arg "$arg"` ;;
       --log-error=*)
        log_error=`parse_arg "$arg"` ;;
+      --user=*)
         # Note that the user will be passed to mysqld so that it runs
         # as 'user' (crucial e.g. if log-bin=/some_other_path/
         # where a chown of datadir won't help)
-      --user=*) user=`parse_arg "$arg"` ;;
-      --group=*) group=`parse_arg "$arg"` ;;
+        user=`parse_arg "$arg"` ;;
       --skip-name-resolve) ip_only=1 ;;
       --verbose) verbose=1 ; silent_startup="" ;;
       --rpm) in_rpm=1 ;;
@@ -394,8 +388,9 @@ create_system_tables2="$srcpkgdatadir/mysql_performance_tables.sql"
 fill_system_tables="$srcpkgdatadir/mysql_system_tables_data.sql"
 maria_add_gis_sp="$buildpkgdatadir/maria_add_gis_sp_bootstrap.sql"
 mysql_test_db="$srcpkgdatadir/mysql_test_db.sql"
+mysql_sys_schema="$buildpkgdatadir/mysql_sys_schema.sql"
 
-for f in "$fill_help_tables" "$create_system_tables" "$create_system_tables2" "$fill_system_tables" "$maria_add_gis_sp" "$mysql_test_db"
+for f in "$fill_help_tables" "$create_system_tables" "$create_system_tables2" "$fill_system_tables" "$maria_add_gis_sp" "$mysql_test_db" "$mysql_sys_schema"
 do
   if test ! -f "$f"
   then
@@ -473,12 +468,7 @@ do
   fi
   if test -n "$user"
   then
-    if test -z "$group"
-    then
-      chown $user $dir
-    else
-      chown $user:$group $dir
-    fi
+    chown $user "$dir"
     if test $? -ne 0
     then
       echo "Cannot change ownership of the database directories to the '$user'"
@@ -512,11 +502,6 @@ then
   args="$args --user=$user"
 fi
 
-if test -n "$group"
-then
-  args="$args --group=$group"
-fi
-
 if test -f "$ldata/mysql/user.frm"
 then
     echo "mysql.user table already exists!"
@@ -540,7 +525,7 @@ mysqld_install_cmd_line()
 {
   "$mysqld_bootstrap" $defaults $defaults_group_suffix "$mysqld_opt" --bootstrap $silent_startup\
   "--basedir=$basedir" "--datadir=$ldata" --log-warnings=0 --enforce-storage-engine="" \
-  "--plugin-dir=${plugindir}" --loose-disable-plugin-file-key-management \
+  "--plugin-dir=${plugindir}" \
   $args --max_allowed_packet=8M \
   --net_buffer_length=16K
 }
@@ -564,7 +549,7 @@ cat_sql()
       ;;
   esac
 
-  cat "$create_system_tables" "$create_system_tables2" "$fill_system_tables" "$fill_help_tables" "$maria_add_gis_sp"
+  cat "$create_system_tables" "$create_system_tables2" "$fill_system_tables" "$fill_help_tables" "$maria_add_gis_sp" "$mysql_sys_schema"
   if test "$skip_test_db" -eq 0
   then
     cat "$mysql_test_db"
@@ -575,7 +560,6 @@ cat_sql()
 s_echo "Installing MariaDB/MySQL system tables in '$ldata' ..."
 if cat_sql | eval "$filter_cmd_line" | mysqld_install_cmd_line > /dev/null
 then
-    printf "@VERSION@-MariaDB" > "$ldata/mysql_upgrade_info"
   s_echo "OK"
 else
   log_file_place=$ldata
@@ -628,8 +612,12 @@ then
     echo
     echo
     echo "PLEASE REMEMBER TO SET A PASSWORD FOR THE MariaDB root USER !"
-    echo "To do so, start the server, then issue the following command:"
+    echo "To do so, start the server, then issue the following commands:"
     echo
+    echo "'$bindir/mysqladmin' -u root password 'new-password'"
+    echo "'$bindir/mysqladmin' -u root -h $hostname password 'new-password'"
+    echo
+    echo "Alternatively you can run:"
     echo "'$bindir/mysql_secure_installation'"
     echo
     echo "which will also give you the option of removing the test"
@@ -648,7 +636,8 @@ then
   fi
 
   echo
-  echo "See the MariaDB Knowledgebase at https://mariadb.com/kb"
+  echo "See the MariaDB Knowledgebase at https://mariadb.com/kb or the"
+  echo "MySQL manual for more instructions."
 
   if test "$in_rpm" -eq 0
   then
@@ -664,7 +653,8 @@ then
   echo "Please report any problems at https://mariadb.org/jira"
   echo
   echo "The latest information about MariaDB is available at https://mariadb.org/."
-  echo
+  echo "You can find additional information about the MySQL part at:"
+  echo "https://dev.mysql.com"
   echo "Consider joining MariaDB's strong and vibrant community:"
   echo "https://mariadb.org/get-involved/"
   echo
