@@ -1,5 +1,5 @@
 /* Copyright (c) 2000, 2016, Oracle and/or its affiliates.
-   Copyright (c) 2012, 2018, MariaDB Corporation.
+   Copyright (c) 2012, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
   embedded library
  */
 
-#include <my_global.h>
+#include "mariadb.h"
 #include <mysql.h>
 #include <mysql_com.h>
 #include <mysqld_error.h>
@@ -45,6 +45,11 @@
 #include <violite.h>
 #include <signal.h>
 #include "probes_mysql.h"
+#include <debug_sync.h>
+#include "proxy_protocol.h"
+
+PSI_memory_key key_memory_NET_buff;
+PSI_memory_key key_memory_NET_compress_packet;
 
 #ifdef EMBEDDED_LIBRARY
 #undef MYSQL_SERVER
@@ -64,8 +69,11 @@ static void inline EXTRA_DEBUG_fprintf(...) {}
 #ifndef MYSQL_SERVER
 static int inline EXTRA_DEBUG_fflush(...) { return 0; }
 #endif
-#endif
+#endif /* EXTRA_DEBUG */
+
 #ifdef MYSQL_SERVER
+#include <sql_class.h>
+#include <sql_connect.h>
 #define MYSQL_SERVER_my_error my_error
 #else
 static void inline MYSQL_SERVER_my_error(...) {}
@@ -106,25 +114,24 @@ void sql_print_error(const char *format,...);
   extern, but as it's hard to include sql_priv.h here, we have to
   live with this for a while.
 */
-extern uint test_flags;
+extern ulonglong test_flags;
 extern ulong bytes_sent, bytes_received, net_big_packet_count;
 #ifdef HAVE_QUERY_CACHE
 #define USE_QUERY_CACHE
-extern void query_cache_insert(void *thd, const char *packet, ulong length,
+extern void query_cache_insert(void *thd, const char *packet, size_t length,
                                unsigned pkt_nr);
 #endif // HAVE_QUERY_CACHE
 #define update_statistics(A) A
-extern my_bool thd_net_is_killed();
+extern my_bool thd_net_is_killed(THD *thd);
 /* Additional instrumentation hooks for the server */
 #include "mysql_com_server.h"
 #else
 #define update_statistics(A)
-#define thd_net_is_killed() 0
+#define thd_net_is_killed(A) 0
 #endif
 
-#define TEST_BLOCKING		8
 
-static my_bool net_write_buff(NET *, const uchar *, ulong);
+static my_bool net_write_buff(NET *, const uchar *, size_t len);
 
 my_bool net_allocate_new_packet(NET *net, void *thd, uint my_flags);
 
@@ -135,6 +142,8 @@ my_bool my_net_init(NET *net, Vio *vio, void *thd, uint my_flags)
   DBUG_ENTER("my_net_init");
   DBUG_PRINT("enter", ("my_flags: %u", my_flags));
   net->vio = vio;
+  net->read_timeout= 0;
+  net->write_timeout= 0;
   my_net_local_init(net);			/* Set some limits */
 
   if (net_allocate_new_packet(net, thd, my_flags))
@@ -173,8 +182,9 @@ my_bool my_net_init(NET *net, Vio *vio, void *thd, uint my_flags)
 my_bool net_allocate_new_packet(NET *net, void *thd, uint my_flags)
 {
   DBUG_ENTER("net_allocate_new_packet");
-  if (!(net->buff=(uchar*) my_malloc((size_t) net->max_packet+
-				     NET_HEADER_SIZE + COMP_HEADER_SIZE +1,
+  if (!(net->buff=(uchar*) my_malloc(key_memory_NET_buff,
+                                     (size_t) net->max_packet +
+				     NET_HEADER_SIZE + COMP_HEADER_SIZE + 1,
 				     MYF(MY_WME | my_flags))))
     DBUG_RETURN(1);
   net->buff_end=net->buff+net->max_packet;
@@ -218,11 +228,11 @@ my_bool net_realloc(NET *net, size_t length)
     my_real_read() may actually read 4 bytes depending on build flags and
     platform.
   */
-  if (!(buff= (uchar*) my_realloc((char*) net->buff, pkt_length +
+  if (!(buff= (uchar*) my_realloc(key_memory_NET_buff,
+                                  (char*) net->buff, pkt_length +
                                   NET_HEADER_SIZE + COMP_HEADER_SIZE + 1,
-                                  MYF(MY_WME |
-                                      (net->thread_specific_malloc ?
-                                       MY_THREAD_SPECIFIC : 0)))))
+                                  MYF(MY_WME | (net->thread_specific_malloc
+                                                ?  MY_THREAD_SPECIFIC : 0)))))
   {
     /* @todo: 1 and 2 codes are identical. */
     net->error= 1;
@@ -404,7 +414,6 @@ my_bool net_flush(NET *net)
 my_bool my_net_write(NET *net, const uchar *packet, size_t len)
 {
   uchar buff[NET_HEADER_SIZE];
-  int rc;
 
   if (unlikely(!net->vio)) /* nowhere to write */
     return 0;
@@ -441,7 +450,7 @@ my_bool my_net_write(NET *net, const uchar *packet, size_t len)
 #ifndef DEBUG_DATA_PACKETS
   DBUG_DUMP("packet_header", buff, NET_HEADER_SIZE);
 #endif
-  rc= MY_TEST(net_write_buff(net, packet, len));
+  my_bool rc= MY_TEST(net_write_buff(net, packet, len));
   MYSQL_NET_WRITE_DONE(rc);
   return rc;
 }
@@ -482,10 +491,21 @@ net_write_command(NET *net,uchar command,
   size_t length=len+1+head_len;			/* 1 extra byte for command */
   uchar buff[NET_HEADER_SIZE+1];
   uint header_size=NET_HEADER_SIZE+1;
-  int rc;
+  my_bool rc;
   DBUG_ENTER("net_write_command");
   DBUG_PRINT("enter",("length: %lu", (ulong) len));
 
+  DBUG_EXECUTE_IF("simulate_error_on_packet_write",
+                  {
+                    if (command == COM_BINLOG_DUMP)
+                    {
+                      net->last_errno = ER_NET_ERROR_ON_WRITE;
+                      DBUG_ASSERT(!debug_sync_set_action(
+                      (THD *)net->thd,
+                      STRING_WITH_LEN("now SIGNAL parked WAIT_FOR continue")));
+                      DBUG_RETURN(true);
+                    }
+                  };);
   MYSQL_NET_WRITE_START(length);
 
   buff[4]=command;				/* For first packet */
@@ -549,13 +569,13 @@ net_write_command(NET *net,uchar command,
 */
 
 static my_bool
-net_write_buff(NET *net, const uchar *packet, ulong len)
+net_write_buff(NET *net, const uchar *packet, size_t len)
 {
-  ulong left_length;
+  size_t left_length;
   if (net->compress && net->max_packet > MAX_PACKET_LENGTH)
-    left_length= (ulong) (MAX_PACKET_LENGTH - (net->write_pos - net->buff));
+    left_length= (MAX_PACKET_LENGTH - (net->write_pos - net->buff));
   else
-    left_length= (ulong) (net->buff_end - net->write_pos);
+    left_length= (net->buff_end - net->write_pos);
 
 #ifdef DEBUG_DATA_PACKETS
   DBUG_DUMP("data_written", packet, len);
@@ -592,7 +612,8 @@ net_write_buff(NET *net, const uchar *packet, ulong len)
       return net_real_write(net, packet, len) ? 1 : 0;
     /* Send out rest of the blocks as full sized blocks */
   }
-  memcpy((char*) net->write_pos,packet,len);
+  if (len)
+    memcpy((char*) net->write_pos,packet,len);
   net->write_pos+= len;
   return 0;
 }
@@ -619,11 +640,23 @@ net_real_write(NET *net,const uchar *packet, size_t len)
   my_bool net_blocking = vio_is_blocking(net->vio);
   DBUG_ENTER("net_real_write");
 
-#if defined(MYSQL_SERVER) && defined(USE_QUERY_CACHE)
-  query_cache_insert(net->thd, (char*) packet, len, net->pkt_nr);
+#if defined(MYSQL_SERVER)
+  THD *thd= (THD *)net->thd;
+#if defined(USE_QUERY_CACHE)
+  query_cache_insert(thd, (char*) packet, len, net->pkt_nr);
+#endif
+  if (likely(thd))
+  {
+    /*
+      Wait until pending operations (currently it is engine
+      asynchronous group commit) are finished before replying
+      to the client, to keep durability promise.
+    */
+    thd->async_state.wait_for_pending_ops();
+  }
 #endif
 
-  if (net->error == 2)
+  if (unlikely(net->error == 2))
     DBUG_RETURN(-1);				/* socket can't be used */
 
   net->reading_or_writing=2;
@@ -633,11 +666,10 @@ net_real_write(NET *net,const uchar *packet, size_t len)
     size_t complen;
     uchar *b;
     uint header_length=NET_HEADER_SIZE+COMP_HEADER_SIZE;
-    if (!(b= (uchar*) my_malloc(len + NET_HEADER_SIZE +
-                                COMP_HEADER_SIZE + 1,
-                                MYF(MY_WME |
-                                    (net->thread_specific_malloc ?
-                                     MY_THREAD_SPECIFIC : 0)))))
+    if (!(b= (uchar*) my_malloc(key_memory_NET_compress_packet,
+                                len + NET_HEADER_SIZE + COMP_HEADER_SIZE + 1,
+                                MYF(MY_WME | (net->thread_specific_malloc
+                                              ? MY_THREAD_SPECIFIC : 0)))))
     {
       net->error= 2;
       net->last_errno= ER_OUT_OF_RESOURCES;
@@ -832,6 +864,70 @@ static my_bool my_net_skip_rest(NET *net, uint32 remain, thr_alarm_t *alarmed,
 #endif /* NO_ALARM */
 
 
+/**
+  Try to parse and process proxy protocol header.
+
+  This function is called in case MySQL packet header cannot be parsed.
+  It checks if proxy header was sent, and that it was send from allowed remote
+  host, as defined by proxy-protocol-networks parameter.
+
+  If proxy header is parsed, then THD and ACL structures and changed to indicate
+  the new peer address and port.
+
+  Note, that proxy header can only be sent either when the connection is established,
+  or as the client reply packet to
+*/
+#undef IGNORE                   /* for Windows */
+typedef enum { RETRY, ABORT, IGNORE} handle_proxy_header_result;
+static handle_proxy_header_result handle_proxy_header(NET *net)
+{
+#if !defined(MYSQL_SERVER) || defined(EMBEDDED_LIBRARY)
+  return IGNORE;
+#else
+  THD *thd= (THD *)net->thd;
+
+  if (!has_proxy_protocol_header(net) || !thd ||
+      thd->get_command() != COM_CONNECT)
+    return IGNORE;
+
+  /*
+    Proxy information found in the first 4 bytes received so far.
+    Read and parse proxy header , change peer ip address and port in THD.
+  */
+  proxy_peer_info peer_info;
+
+  if (!thd->net.vio)
+  {
+    DBUG_ASSERT(0);
+    return ABORT;
+  }
+
+  if (!is_proxy_protocol_allowed((sockaddr *)&(thd->net.vio->remote)))
+  {
+     /* proxy-protocol-networks variable needs to be set to allow this remote address */
+     my_printf_error(ER_HOST_NOT_PRIVILEGED, "Proxy header is not accepted from %s",
+       MYF(0), thd->main_security_ctx.ip);
+     return ABORT;
+  }
+
+  if (parse_proxy_protocol_header(net, &peer_info))
+  {
+     /* Failed to parse proxy header*/
+     my_printf_error(ER_UNKNOWN_ERROR, "Failed to parse proxy header", MYF(0));
+     return ABORT;
+  }
+
+  if (peer_info.is_local_command)
+    /* proxy header indicates LOCAL connection, no action necessary */
+    return RETRY;
+  /* Change peer address in THD and ACL structures.*/
+  uint host_errors;
+  return (handle_proxy_header_result)thd_set_peer_addr(thd,
+                         &(peer_info.peer_addr), NULL, peer_info.port,
+                         false, &host_errors);
+#endif
+}
+
 /**
   Reads one packet to net->buff + net->where_b.
   Long packets are handled by my_net_read().
@@ -854,6 +950,9 @@ my_real_read(NET *net, size_t *complen,
 #ifndef NO_ALARM
   ALARM alarm_buff;
 #endif
+
+retry:
+
   my_bool net_blocking=vio_is_blocking(net->vio);
   uint32 remain= (net->compress ? NET_HEADER_SIZE+COMP_HEADER_SIZE :
 		  NET_HEADER_SIZE);
@@ -896,7 +995,7 @@ my_real_read(NET *net, size_t *complen,
 	  DBUG_PRINT("info",("vio_read returned %ld  errno: %d",
 			     (long) length, vio_errno(net->vio)));
 
-          if (i== 0 && thd_net_is_killed())
+          if (i== 0 && unlikely(thd_net_is_killed((THD*) net->thd)))
           {
             DBUG_PRINT("info", ("thd is killed"));
             len= packet_error;
@@ -974,7 +1073,7 @@ my_real_read(NET *net, size_t *complen,
 #endif
       if (i == 0)
       {					/* First parts is packet length */
-	ulong helping;
+        size_t helping;
 #ifndef DEBUG_DATA_PACKETS
         DBUG_DUMP("packet_header", net->buff+net->where_b,
                   NET_HEADER_SIZE);
@@ -1085,6 +1184,17 @@ end:
 
 packets_out_of_order:
   {
+    switch (handle_proxy_header(net)) {
+    case ABORT:
+        /* error happened, message is already written. */
+        len= packet_error;
+        goto end; 
+    case RETRY:
+        goto retry;
+    case IGNORE:
+        break;
+    }
+
     DBUG_PRINT("error",
                ("Packets out of order (Found: %d, expected %u)",
                 (int) net->buff[net->where_b + 3],
@@ -1167,22 +1277,23 @@ my_net_read_packet_reallen(NET *net, my_bool read_from_server, ulong* reallen)
       size_t total_length= 0;
       do
       {
-	net->where_b += len;
+	net->where_b += (ulong)len;
 	total_length += len;
 	len = my_real_read(net,&complen, 0);
       } while (len == MAX_PACKET_LENGTH);
-      if (len != packet_error)
+      if (likely(len != packet_error))
 	len+= total_length;
       net->where_b = save_pos;
     }
+
     net->read_pos = net->buff + net->where_b;
-    if (len != packet_error)
+    if (likely(len != packet_error))
     {
       net->read_pos[len]=0;		/* Safeguard for mysql_use_result */
-      *reallen = len;
+      *reallen = (ulong)len;
     }
     MYSQL_NET_READ_DONE(0, len);
-    return len;
+    return (ulong)len;
 #ifdef HAVE_COMPRESS
   }
   else
@@ -1280,7 +1391,7 @@ my_net_read_packet_reallen(NET *net, my_bool read_from_server, ulong* reallen)
         MYSQL_NET_READ_DONE(1, 0);
 	return packet_error;
       }
-      buf_length+= complen;
+      buf_length+= (ulong)complen;
       *reallen += packet_len;
     }
 
@@ -1294,7 +1405,7 @@ my_net_read_packet_reallen(NET *net, my_bool read_from_server, ulong* reallen)
   }
 #endif /* HAVE_COMPRESS */
   MYSQL_NET_READ_DONE(0, len);
-  return len;
+  return (ulong)len;
 }
 
 
