@@ -1168,9 +1168,6 @@ static bool should_send_column_info(THD* thd, List<Item>* list, uint flags)
 */
 bool Protocol::send_result_set_metadata(List<Item> *list, uint flags)
 {
-  List_iterator_fast<Item> it(*list);
-  Item *item;
-  Protocol_text prot(thd, thd->variables.net_buffer_length);
   DBUG_ENTER("Protocol::send_result_set_metadata");
 
   bool send_column_info= should_send_column_info(thd, list, flags);
@@ -1195,6 +1192,9 @@ bool Protocol::send_result_set_metadata(List<Item> *list, uint flags)
 
   if (send_column_info)
   {
+    List_iterator_fast<Item> it(*list);
+    Item *item;
+    Protocol_text prot(thd, thd->variables.net_buffer_length);
 #ifndef DBUG_OFF
     field_handlers= (const Type_handler **) thd->alloc(
         sizeof(field_handlers[0]) * list->elements);
@@ -1316,18 +1316,12 @@ bool Protocol_text::store_field_metadata_for_list_fields(const THD *thd,
 bool Protocol::send_result_set_row(List<Item> *row_items)
 {
   List_iterator_fast<Item> it(*row_items);
-
+  ValueBuffer<MAX_FIELD_WIDTH> value_buffer;
   DBUG_ENTER("Protocol::send_result_set_row");
 
   for (Item *item= it++; item; item= it++)
   {
-    /*
-      ValueBuffer::m_string can be altered during Item::send().
-      It's important to declare value_buffer inside the loop,
-      to have ValueBuffer::m_string point to ValueBuffer::buffer
-      on every iteration.
-    */
-    ValueBuffer<MAX_FIELD_WIDTH> value_buffer;
+    value_buffer.reset_buffer();
     if (item->send(this, &value_buffer))
     {
       // If we're out of memory, reclaim some, to help us recover.
@@ -1344,9 +1338,9 @@ bool Protocol::send_result_set_row(List<Item> *row_items)
 
 
 /**
-  Send \\0 end terminated string.
+  Send \\0 end terminated string or NULL
 
-  @param from	NullS or \\0 terminated string
+  @param from    NullS or \\0 terminated string
 
   @note
     In most cases one should use store(from, length) instead of this function
@@ -1357,12 +1351,11 @@ bool Protocol::send_result_set_row(List<Item> *row_items)
     1		error
 */
 
-bool Protocol::store(const char *from, CHARSET_INFO *cs)
+bool Protocol::store_string_or_null(const char *from, CHARSET_INFO *cs)
 {
   if (!from)
     return store_null();
-  size_t length= strlen(from);
-  return store(from, length, cs);
+  return store(from, strlen(from), cs);
 }
 
 
@@ -1381,7 +1374,7 @@ bool Protocol::store(I_List<i_string>* str_list)
   tmp.length(0);
   while ((s=it++))
   {
-    tmp.append(s->ptr);
+    tmp.append(s->ptr, strlen(s->ptr));
     tmp.append(',');
   }
   if ((len= tmp.length()))
