@@ -533,7 +533,6 @@ static my_old_conv old_conv[]=
 CHARSET_INFO *get_old_charset_by_name(const char *name)
 {
   my_old_conv *conv;
-
   for (conv= old_conv; conv->old_name; conv++)
   {
     if (!my_strcasecmp(&my_charset_latin1, name, conv->old_name))
@@ -1034,7 +1033,7 @@ int set_var_collation_client::check(THD *thd)
   if (!is_supported_parser_charset(character_set_client))
   {
     my_error(ER_WRONG_VALUE_FOR_VAR, MYF(0), "character_set_client",
-             character_set_client->csname);
+             character_set_client->cs_name.str);
     return 1;
   }
   return 0;
@@ -1219,12 +1218,14 @@ int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
     {
       uint i;
       strbuf.length(0);
-      for (i=0; i + 1 < tl->count; i++)
+      for (i=0; i < tl->count; i++)
       {
-        strbuf.append(tl->type_names[i]);
+        const char *name= tl->type_names[i];
+        strbuf.append(name, strlen(name));
         strbuf.append(',');
       }
-      strbuf.append(tl->type_names[i]);
+      if (!strbuf.is_empty())
+        strbuf.chop();
       fields[11]->set_notnull();
       fields[11]->store(strbuf.ptr(), strbuf.length(), scs);
     }
