@@ -798,7 +798,7 @@ bool thd_init_client_charset(THD *thd, uint cs_number)
     {
       /* Disallow non-supported parser character sets: UCS2, UTF16, UTF32 */
       my_error(ER_WRONG_VALUE_FOR_VAR, MYF(0), "character_set_client",
-               cs->csname);
+               cs->cs_name.str);
       return true;
     }
     thd->org_charset= cs;
@@ -1108,12 +1108,6 @@ static int check_connection(THD *thd)
 
 void setup_connection_thread_globals(THD *thd)
 {
-  DBUG_EXECUTE_IF("CONNECT_wait", {
-    extern MYSQL_SOCKET unix_sock;
-    DBUG_ASSERT(unix_sock.fd >= 0);
-    while (unix_sock.fd >= 0)
-      my_sleep(1000);
-  });
   thd->store_globals();
 }
 
@@ -1378,10 +1372,10 @@ void do_handle_one_connection(CONNECT *connect, bool put_in_cache)
     if (launch_time >= slow_launch_time*1000000L)
       statistic_increment(slow_launch_threads, &LOCK_status);
   }
+  delete connect;
 
-  server_threads.insert(thd); // Make THD visible in show processlist
-
-  delete connect; // must be after server_threads.insert, see close_connections()
+  /* Make THD visible in show processlist */
+  server_threads.insert(thd);
   
   thd->thr_create_utime= thr_create_utime;
   /* We need to set this because of time_out_user_resource_limits */
