@@ -73,7 +73,6 @@ bool
 mysql_handle_derived(LEX *lex, uint phases)
 {
   bool res= FALSE;
-  THD *thd= lex->thd;
   DBUG_ENTER("mysql_handle_derived");
   DBUG_PRINT("enter", ("phases: 0x%x", phases));
   if (!lex->derived_tables)
@@ -88,8 +87,6 @@ mysql_handle_derived(LEX *lex, uint phases)
       break;
     if (!(phases & phase_flag))
       continue;
-    if (phase_flag >= DT_CREATE && !thd->fill_derived_tables())
-      break;
 
     for (SELECT_LEX *sl= lex->all_selects_list;
 	 sl && !res;
@@ -173,7 +170,6 @@ bool
 mysql_handle_single_derived(LEX *lex, TABLE_LIST *derived, uint phases)
 {
   bool res= FALSE;
-  THD *thd= lex->thd;
   uint8 allowed_phases= (derived->is_merged_derived() ? DT_PHASES_MERGE :
                          DT_PHASES_MATERIALIZE);
   DBUG_ENTER("mysql_handle_single_derived");
@@ -200,8 +196,6 @@ mysql_handle_single_derived(LEX *lex, TABLE_LIST *derived, uint phases)
     if (phase_flag != DT_PREPARE &&
         !(allowed_phases & phase_flag))
       continue;
-    if (phase_flag >= DT_CREATE && !thd->fill_derived_tables())
-      break;
 
     if ((res= (*processors[phase])(lex->thd, lex, derived)))
       break;
@@ -382,10 +376,6 @@ bool mysql_derived_merge(THD *thd, LEX *lex, TABLE_LIST *derived)
     DBUG_RETURN(FALSE);
   }
 
-  if (thd->lex->sql_command == SQLCOM_UPDATE_MULTI ||
-      thd->lex->sql_command == SQLCOM_DELETE_MULTI)
-    thd->save_prep_leaf_list= TRUE;
-
   arena= thd->activate_stmt_arena_if_needed(&backup);  // For easier test
 
   if (!derived->merged_for_insert || 
@@ -459,6 +449,7 @@ bool mysql_derived_merge(THD *thd, LEX *lex, TABLE_LIST *derived)
       derived->on_expr= expr;
       derived->prep_on_expr= expr->copy_andor_structure(thd);
     }
+    thd->where= "on clause";
     if (derived->on_expr &&
         derived->on_expr->fix_fields_if_needed_for_bool(thd, &derived->on_expr))
     {
@@ -682,14 +673,29 @@ static
 bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
   SELECT_LEX_UNIT *unit= derived->get_unit();
-  bool res= FALSE;
+  SELECT_LEX *first_select;
+  bool res= FALSE, keep_row_order;
   DBUG_ENTER("mysql_derived_prepare");
   DBUG_PRINT("enter", ("unit: %p  table_list: %p  alias: '%s'",
                        unit, derived, derived->alias.str));
   if (!unit)
     DBUG_RETURN(FALSE);
 
-  SELECT_LEX *first_select= unit->first_select();
+  first_select= unit->first_select();
+  /*
+    If rownum() is used we have to preserve the insert row order
+    to make GROUP BY and ORDER BY with filesort work.
+
+    SELECT * from (SELECT a,b from t1 ORDER BY a)) WHERE rownum <= 0;
+
+    When rownum is not used the optimizer will skip the ORDER BY clause.
+    With rownum we have to keep the ORDER BY as this is what is expected.
+    We also have to create any sort result temporary table in such a way
+    that the inserted row order is maintained.
+  */
+  keep_row_order= (thd->lex->with_rownum &&
+                   (first_select->group_list.elements ||
+                    first_select->order_list.elements));
 
   if (derived->is_recursive_with_table() &&
       !derived->is_with_table_recursive_reference() &&
@@ -726,7 +732,8 @@ bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
                                   (first_select->options |
                                    thd->variables.option_bits |
                                    TMP_TABLE_ALL_COLUMNS),
-                                  &derived->alias, FALSE, FALSE, FALSE, 0);
+                                  &derived->alias, FALSE, FALSE,
+                                  keep_row_order, 0);
     thd->create_tmp_table_for_derived= FALSE;
 
     if (likely(!res) && !derived->table)
@@ -879,7 +886,7 @@ bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
                                                    thd->variables.option_bits |
                                                    TMP_TABLE_ALL_COLUMNS),
                                                    &derived->alias,
-                                                   FALSE, FALSE, FALSE,
+                                                   FALSE, FALSE, keep_row_order,
                                                    0))
   { 
     thd->create_tmp_table_for_derived= FALSE;
