@@ -297,10 +297,10 @@ int get_topics_for_keyword(THD *thd, TABLE *topics, TABLE *relations,
   DBUG_ENTER("get_topics_for_keyword");
 
   if ((iindex_topic=
-       find_type(primary_key_name, &topics->s->keynames,
+       find_type(primary_key_name.str, &topics->s->keynames,
                  FIND_TYPE_NO_PREFIX) - 1) < 0 ||
       (iindex_relations=
-       find_type(primary_key_name, &relations->s->keynames,
+       find_type(primary_key_name.str, &relations->s->keynames,
                  FIND_TYPE_NO_PREFIX) - 1) < 0)
   {
     my_message(ER_CORRUPT_HELP_DB, ER_THD(thd, ER_CORRUPT_HELP_DB), MYF(0));
@@ -541,7 +541,12 @@ extern "C" int string_ptr_cmp(const void* ptr1, const void* ptr2)
 {
   String *str1= *(String**)ptr1;
   String *str2= *(String**)ptr2;
-  return strcmp(str1->c_ptr(),str2->c_ptr());
+  uint length1= str1->length();
+  uint length2= str2->length();
+  int tmp= memcmp(str1->ptr(),str2->ptr(), MY_MIN(length1, length2));
+  if (tmp)
+    return tmp;
+  return (int) length2 - (int) length1;
 }
 
 /*
