@@ -592,15 +592,9 @@ int compare_window_funcs_by_window_specs(Item_window_func *win_func1,
       Let's use only one of the lists.
     */
     if (!win_spec1->name() && win_spec2->name())
-    {
-      win_spec1->save_partition_list= win_spec1->partition_list;
       win_spec1->partition_list= win_spec2->partition_list;
-    }
     else
-    {
-      win_spec2->save_partition_list= win_spec2->partition_list;
       win_spec2->partition_list= win_spec1->partition_list;
-    }
 
     cmp= compare_order_lists(win_spec1->order_list,
                              win_spec2->order_list);
@@ -613,15 +607,9 @@ int compare_window_funcs_by_window_specs(Item_window_func *win_func1,
        Let's use only one of the lists.
     */
     if (!win_spec1->name() && win_spec2->name())
-    {
-      win_spec1->save_order_list= win_spec2->order_list;
       win_spec1->order_list= win_spec2->order_list;
-    }
     else
-    {
-      win_spec1->save_order_list= win_spec2->order_list;
       win_spec2->order_list= win_spec1->order_list;
-    }
 
     cmp= compare_window_frames(win_spec1->window_frame,
                                win_spec2->window_frame);
@@ -790,10 +778,10 @@ public:
     {
       //DBUG_ASSERT(info->read_record == rr_from_tempfile);
       rownum= 0;
-      io_cache= (IO_CACHE*)my_malloc(sizeof(IO_CACHE), MYF(0));
+      io_cache= (IO_CACHE*)my_malloc(PSI_INSTRUMENT_ME, sizeof(IO_CACHE), MYF(0));
       init_slave_io_cache(info->io_cache, io_cache);
 
-      ref_buffer= (uchar*)my_malloc(ref_length, MYF(0));
+      ref_buffer= (uchar*)my_malloc(PSI_INSTRUMENT_ME, ref_length, MYF(0));
       ref_buffer_valid= false;
     }
   }
@@ -2509,7 +2497,7 @@ void add_special_frame_cursors(THD *thd, Cursor_manager *cursor_manager,
       cursor_manager->add_cursor(bottom_bound);
       cursor_manager->add_cursor(top_bound);
       cursor_manager->add_cursor(current_row_pos);
-      DBUG_ASSERT(item_sum->fixed);
+      DBUG_ASSERT(item_sum->fixed());
       bool negative_offset= item_sum->sum_func() == Item_sum::LAG_FUNC;
       fc= new Frame_positional_cursor(*current_row_pos,
                                       *top_bound, *bottom_bound,
@@ -2525,7 +2513,7 @@ void add_special_frame_cursors(THD *thd, Cursor_manager *cursor_manager,
       Frame_cursor *top_bound= get_frame_cursor(thd, spec, true);
       cursor_manager->add_cursor(bottom_bound);
       cursor_manager->add_cursor(top_bound);
-      DBUG_ASSERT(item_sum->fixed);
+      DBUG_ASSERT(item_sum->fixed());
       Item *offset_item= new (thd->mem_root) Item_int(thd, 0);
       offset_item->fix_fields(thd, &offset_item);
       fc= new Frame_positional_cursor(*top_bound,
@@ -2541,7 +2529,7 @@ void add_special_frame_cursors(THD *thd, Cursor_manager *cursor_manager,
       Frame_cursor *top_bound= get_frame_cursor(thd, spec, true);
       cursor_manager->add_cursor(bottom_bound);
       cursor_manager->add_cursor(top_bound);
-      DBUG_ASSERT(item_sum->fixed);
+      DBUG_ASSERT(item_sum->fixed());
       Item *offset_item= new (thd->mem_root) Item_int(thd, 0);
       offset_item->fix_fields(thd, &offset_item);
       fc= new Frame_positional_cursor(*bottom_bound,
@@ -2557,7 +2545,7 @@ void add_special_frame_cursors(THD *thd, Cursor_manager *cursor_manager,
       Frame_cursor *top_bound= get_frame_cursor(thd, spec, true);
       cursor_manager->add_cursor(bottom_bound);
       cursor_manager->add_cursor(top_bound);
-      DBUG_ASSERT(item_sum->fixed);
+      DBUG_ASSERT(item_sum->fixed());
       Item *int_item= new (thd->mem_root) Item_int(thd, 1);
       Item *offset_func= new (thd->mem_root)
                               Item_func_minus(thd, item_sum->get_arg(1),
@@ -2746,7 +2734,7 @@ bool save_window_function_values(List<Item_window_func>& window_functions,
   Item *func;
   for (; (func = *func_ptr) ; func_ptr++)
   {
-    if (func->with_window_func && func->type() != Item::WINDOW_FUNC_ITEM)
+    if (func->with_window_func() && func->type() != Item::WINDOW_FUNC_ITEM)
       func->save_in_result_field(true);
   }
 
@@ -2828,7 +2816,7 @@ bool compute_window_func(THD *thd,
 
   List_iterator_fast<Group_bound_tracker> iter_part_trackers(partition_trackers);
   ha_rows rownum= 0;
-  uchar *rowid_buf= (uchar*) my_malloc(tbl->file->ref_length, MYF(0));
+  uchar *rowid_buf= (uchar*) my_malloc(PSI_INSTRUMENT_ME, tbl->file->ref_length, MYF(0));
 
   while (true)
   {
@@ -2948,6 +2936,14 @@ bool Window_func_runner::add_function_to_run(Item_window_func *win_func)
       my_error(ER_NOT_SUPPORTED_YET, MYF(0),
                "COUNT(DISTINCT) aggregate as window function");
       return true;
+    case Item_sum::JSON_ARRAYAGG_FUNC:
+      my_error(ER_NOT_SUPPORTED_YET, MYF(0),
+               "JSON_ARRAYAGG() aggregate as window function");
+      return true;
+    case Item_sum::JSON_OBJECTAGG_FUNC:
+      my_error(ER_NOT_SUPPORTED_YET, MYF(0),
+               "JSON_OBJECTAGG() aggregate as window function");
+      return true;
     default:
       break;
   }
@@ -2968,7 +2964,8 @@ bool Window_func_runner::exec(THD *thd, TABLE *tbl, SORT_INFO *filesort_result)
     win_func->set_phase_to_computation();
     // TODO(cvicentiu) Setting the aggregator should probably be done during
     // setup of Window_funcs_sort.
-    win_func->window_func()->set_aggregator(Aggregator::SIMPLE_AGGREGATOR);
+    win_func->window_func()->set_aggregator(thd,
+                                            Aggregator::SIMPLE_AGGREGATOR);
   }
   it.rewind();
 
