@@ -29,8 +29,8 @@
 #include <sql_plist.h>
 #include <threadpool.h>
 #include <algorithm>
-
-#ifdef HAVE_IOCP
+#ifdef _WIN32
+#include "threadpool_winsockets.h"
 #define OPTIONAL_IO_POLL_READ_PARAM this
 #else 
 #define OPTIONAL_IO_POLL_READ_PARAM 0
@@ -234,19 +234,14 @@ static void *native_event_get_userdata(native_event *event)
 #elif defined(HAVE_KQUEUE)
 
 /*
-  NetBSD prior to 9.99.17 is incompatible with other BSDs, last parameter
-  in EV_SET macro (udata, user data) needs to be intptr_t, whereas it needs
-  to be void* everywhere else.
+  NetBSD is incompatible with other BSDs , last parameter in EV_SET macro
+  (udata, user data) needs to be intptr_t, whereas it needs to be void*
+  everywhere else.
 */
 
 #ifdef __NetBSD__
-#include <sys/param.h>
-#  if !__NetBSD_Prereq__(9,99,17)
 #define MY_EV_SET(a, b, c, d, e, f, g) EV_SET(a, b, c, d, e, f, (intptr_t)g)
-#  endif
-#endif
-
-#ifndef MY_EV_SET
+#else
 #define MY_EV_SET(a, b, c, d, e, f, g) EV_SET(a, b, c, d, e, f, g)
 #endif
 
@@ -352,7 +347,7 @@ static void* native_event_get_userdata(native_event *event)
   return event->portev_user;
 }
 
-#elif defined(HAVE_IOCP)
+#elif defined(_WIN32)
 
 
 static TP_file_handle io_poll_create()
@@ -363,29 +358,8 @@ static TP_file_handle io_poll_create()
 
 int io_poll_start_read(TP_file_handle pollfd, TP_file_handle fd, void *, void *opt)
 {
-  static char c;
-  TP_connection_generic *con= (TP_connection_generic *)opt;
-  OVERLAPPED *overlapped= &con->overlapped;
-  if (con->vio_type == VIO_TYPE_NAMEDPIPE)
-  {
-    if (ReadFile(fd, &c, 0, NULL, overlapped))
-      return 0;
-  }
-  else
-  {
-    WSABUF buf;
-    buf.buf= &c;
-    buf.len= 0;
-    DWORD flags=0;
-
-    if (WSARecv((SOCKET)fd, &buf, 1,NULL, &flags,overlapped, NULL) == 0)
-      return 0;
-  }
-
-  if (GetLastError() == ERROR_IO_PENDING)
-    return 0;
-
-  return 1;
+  auto c= (TP_connection_generic *) opt;
+  return (int) c->win_sock.begin_read();
 }
 
 
@@ -434,20 +408,33 @@ int io_poll_disassociate_fd(TP_file_handle pollfd, TP_file_handle fd)
 }
 
 
-int io_poll_wait(TP_file_handle pollfd, native_event *events, int maxevents, int timeout_ms)
+static void *native_event_get_userdata(native_event *event)
+{
+  return (void *) event->lpCompletionKey;
+}
+
+int io_poll_wait(TP_file_handle pollfd, native_event *events, int maxevents,
+                 int timeout_ms)
 {
   ULONG n;
-  BOOL ok = GetQueuedCompletionStatusEx(pollfd, events,
-     maxevents, &n, timeout_ms, FALSE);
+  if (!GetQueuedCompletionStatusEx(pollfd, events, maxevents, &n, timeout_ms, FALSE))
+    return -1;
 
-  return ok ? (int)n : -1;
+  /* Update win_sock with number of bytes read.*/
+  for (ULONG i= 0; i < n; i++)
+  {
+    auto ev= &events[i];
+    auto c= (TP_connection_generic *) native_event_get_userdata(ev);
+    /* null userdata zero means shutdown (see PostQueuedCompletionStatus() usage*/
+    if (c)
+    {
+      c->win_sock.end_read(ev->dwNumberOfBytesTransferred, 0);
+    }
+  }
+
+  return (int) n;
 }
 
-
-static void* native_event_get_userdata(native_event *event)
-{
-  return (void *)event->lpCompletionKey;
-}
 #endif
 
 
@@ -1010,7 +997,7 @@ void thread_group_destroy(thread_group_t *thread_group)
     io_poll_close(thread_group->pollfd);
     thread_group->pollfd= INVALID_HANDLE_VALUE;
   }
-#ifndef HAVE_IOCP
+#ifndef _WIN32
   for(int i=0; i < 2; i++)
   {
     if(thread_group->shutdown_pipe[i] != -1)
@@ -1057,7 +1044,7 @@ static int wake_thread(thread_group_t *thread_group,bool due_to_stall)
 */
 static int wake_listener(thread_group_t *thread_group)
 {
-#ifndef HAVE_IOCP
+#ifndef _WIN32
   if (pipe(thread_group->shutdown_pipe))
   {
     return -1;
@@ -1214,7 +1201,7 @@ TP_connection_generic *get_event(worker_thread_t *current_thread,
       non-blocking event poll, i.e with timeout = 0.
       If this returns events, pick one
     */
-    if (!oversubscribed && !threadpool_dedicated_listener)
+    if (!oversubscribed)
     {
       native_event ev[MAX_EVENTS];
       int cnt = io_poll_wait(thread_group->pollfd, ev, MAX_EVENTS, 0);
@@ -1340,7 +1327,10 @@ void TP_pool_generic::add(TP_connection *c)
   DBUG_VOID_RETURN;
 }
 
-
+void TP_pool_generic::resume(TP_connection* c)
+{
+  add(c);
+}
 
 /**
   MySQL scheduler callback: wait begin
@@ -1403,12 +1393,6 @@ TP_connection_generic::TP_connection_generic(CONNECT *c):
   bound_to_poll_descriptor(false),
   waiting(false),
   fix_group(false)
-#ifdef HAVE_IOCP
-, overlapped()
-#endif
-#ifdef _WIN32
-, vio_type(c->vio_type)
-#endif
 {
   DBUG_ASSERT(c->vio_type != VIO_CLOSED);
 
@@ -1601,9 +1585,6 @@ int TP_pool_generic::init()
     sql_print_error("Allocation failed");
     DBUG_RETURN(-1);
   }
-  PSI_register(mutex);
-  PSI_register(cond);
-  PSI_register(thread);
   scheduler_init();
   threadpool_started= true;
   for (uint i= 0; i < threadpool_max_size; i++)
@@ -1617,6 +1598,10 @@ int TP_pool_generic::init()
     sql_print_error("Can't set threadpool size to %d",threadpool_size);
     DBUG_RETURN(-1);
   }
+  PSI_register(mutex);
+  PSI_register(cond);
+  PSI_register(thread);
+
   pool_timer.tick_interval= threadpool_stall_limit;
   start_timer(&pool_timer);
   DBUG_RETURN(0);
@@ -1757,9 +1742,9 @@ static void print_pool_blocked_message(bool max_threads_reached)
   if (now > pool_block_start + BLOCK_MSG_DELAY && !msg_written)
   {
     if (max_threads_reached)
-      sql_print_warning(MAX_THREADS_REACHED_MSG);
+      sql_print_error(MAX_THREADS_REACHED_MSG);
     else
-      sql_print_warning(CREATE_THREAD_ERROR_MSG, my_errno);
+      sql_print_error(CREATE_THREAD_ERROR_MSG, my_errno);
 
     sql_print_information("Threadpool has been blocked for %u seconds\n",
       (uint)((now- pool_block_start)/1000000));
