@@ -104,7 +104,7 @@ typedef struct st_federatedx_share {
   /*
     the primary select query to be used in rnd_init
   */
-  char *select_query;
+  LEX_CSTRING select_query;
   /*
     remote host info, parse_url supplies
   */
@@ -262,7 +262,7 @@ public:
 /*
   Class definition for the storage engine
 */
-class ha_federatedx: public handler
+class ha_federatedx final : public handler
 {
   friend int federatedx_db_init(void *p);
 
@@ -338,7 +338,7 @@ public:
             | HA_REC_NOT_IN_SEQ | HA_AUTO_PART_KEY | HA_CAN_INDEX_BLOBS |
             HA_BINLOG_ROW_CAPABLE | HA_BINLOG_STMT_CAPABLE | HA_CAN_REPAIR |
             HA_PRIMARY_KEY_REQUIRED_FOR_DELETE | HA_CAN_ONLINE_BACKUPS |
-            HA_PARTIAL_COLUMN_READ | HA_NULL_IN_KEY | HA_NON_COMPARABLE_ROWID);
+            HA_PARTIAL_COLUMN_READ | HA_NULL_IN_KEY);
   }
   /*
     This is a bitmap of flags that says how the storage engine
@@ -428,31 +428,21 @@ public:
   int rnd_next(uchar *buf);                                      //required
   int rnd_pos(uchar *buf, uchar *pos);                            //required
   void position(const uchar *record);                            //required
-  /*
-    A ref is a pointer inside a local buffer. It is not comparable to
-    other ref's. This is never called as HA_NON_COMPARABLE_ROWID is set.
-  */
-  int cmp_ref(const uchar *ref1, const uchar *ref2)
-  {
-#ifdef NOT_YET
-    DBUG_ASSERT(0);
-    return 0;
-#else
-    return handler::cmp_ref(ref1,ref2);         /* Works if table scan is used */
-#endif
-  }
   int info(uint);                                              //required
   int extra(ha_extra_function operation);
 
   void update_auto_increment(void);
   int repair(THD* thd, HA_CHECK_OPT* check_opt);
   int optimize(THD* thd, HA_CHECK_OPT* check_opt);
-
+  int delete_table(const char *name)
+  {
+    return 0;
+  }
   int delete_all_rows(void);
   int create(const char *name, TABLE *form,
              HA_CREATE_INFO *create_info);                      //required
-  ha_rows records_in_range(uint inx, key_range *start_key,
-                                   key_range *end_key);
+  ha_rows records_in_range(uint inx, const key_range *start_key,
+                           const key_range *end_key, page_range *pages);
   uint8 table_cache_type() { return HA_CACHE_TBL_NOCACHE; }
 
   THR_LOCK_DATA **store_lock(THD *thd, THR_LOCK_DATA **to,
@@ -463,7 +453,6 @@ public:
   int reset(void);
   int free_result(void);
 
-  const FEDERATEDX_SHARE *get_federatedx_share() const { return share; }
   friend class ha_federatedx_derived_handler;
   friend class ha_federatedx_select_handler;
 };
