@@ -198,89 +198,6 @@ name_of_col_is(
 }
 #endif /* UNIV_DEBUG */
 
-/********************************************************************//**
-Finds the first table name in the given database.
-@return own: table name, NULL if does not exist; the caller must free
-the memory in the string! */
-char*
-dict_get_first_table_name_in_db(
-/*============================*/
-	const char*	name)	/*!< in: database name which ends in '/' */
-{
-	dict_table_t*	sys_tables;
-	btr_pcur_t	pcur;
-	dict_index_t*	sys_index;
-	dtuple_t*	tuple;
-	mem_heap_t*	heap;
-	dfield_t*	dfield;
-	const rec_t*	rec;
-	const byte*	field;
-	ulint		len;
-	mtr_t		mtr;
-
-	dict_sys.assert_locked();
-
-	heap = mem_heap_create(1000);
-
-	mtr_start(&mtr);
-
-	sys_tables = dict_table_get_low("SYS_TABLES");
-	sys_index = UT_LIST_GET_FIRST(sys_tables->indexes);
-	ut_ad(!dict_table_is_comp(sys_tables));
-
-	tuple = dtuple_create(heap, 1);
-	dfield = dtuple_get_nth_field(tuple, 0);
-
-	dfield_set_data(dfield, name, strlen(name));
-	dict_index_copy_types(tuple, sys_index, 1);
-
-	btr_pcur_open_on_user_rec(sys_index, tuple, PAGE_CUR_GE,
-				  BTR_SEARCH_LEAF, &pcur, &mtr);
-loop:
-	rec = btr_pcur_get_rec(&pcur);
-
-	if (!btr_pcur_is_on_user_rec(&pcur)) {
-		/* Not found */
-
-		btr_pcur_close(&pcur);
-		mtr_commit(&mtr);
-		mem_heap_free(heap);
-
-		return(NULL);
-	}
-
-	field = rec_get_nth_field_old(
-		rec, DICT_FLD__SYS_TABLES__NAME, &len);
-
-	if (len < strlen(name)
-	    || memcmp(name, field, strlen(name))) {
-		/* Not found */
-
-		btr_pcur_close(&pcur);
-		mtr_commit(&mtr);
-		mem_heap_free(heap);
-
-		return(NULL);
-	}
-
-	if (!rec_get_deleted_flag(rec, 0)) {
-
-		/* We found one */
-
-		char*	table_name = mem_strdupl((char*) field, len);
-
-		btr_pcur_close(&pcur);
-		mtr_commit(&mtr);
-		mem_heap_free(heap);
-
-		return(table_name);
-	}
-
-	btr_pcur_move_to_next_user_rec(&pcur, &mtr);
-
-	goto loop;
-}
-
 /********************************************************************//**
 This function gets the next system table record as it scans the table.
 @return the next record if found, NULL if end of scan */
@@ -1029,7 +946,7 @@ next:
 			continue;
 		}
 
-		if (strstr(table_name.m_name, "/" TEMP_FILE_PREFIX "-")) {
+		if (strstr(table_name.m_name, "/" TEMP_FILE_PREFIX_INNODB)) {
 			/* This table will be dropped by
 			row_mysql_drop_garbage_tables().
 			We do not care if the file exists. */
@@ -1180,11 +1097,14 @@ err_len:
 
 	pos = mach_read_from_4(field);
 
-	rec_get_nth_field_offs_old(
+	field = rec_get_nth_field_old(
 		rec, DICT_FLD__SYS_COLUMNS__DB_TRX_ID, &len);
 	if (len != DATA_TRX_ID_LEN && len != UNIV_SQL_NULL) {
 		goto err_len;
 	}
+
+	const trx_id_t trx_id = mach_read_from_6(field);
+
 	rec_get_nth_field_offs_old(
 		rec, DICT_FLD__SYS_COLUMNS__DB_ROLL_PTR, &len);
 	if (len != DATA_ROLL_PTR_LEN && len != UNIV_SQL_NULL) {
@@ -1271,6 +1191,10 @@ err_len:
 			dict_mem_table_add_col(table, heap, name, mtype,
 					       prtype, col_len);
 		}
+
+		if (trx_id > table->def_trx_id) {
+			table->def_trx_id = trx_id;
+		}
 	} else {
 		dict_mem_fill_column_struct(column, pos, mtype,
 					    prtype, col_len);
@@ -1837,10 +1761,6 @@ dict_load_index_low(
 		*index = NULL;
 	}
 
-	if (rec_get_deleted_flag(rec, 0)) {
-		return(dict_load_index_del);
-	}
-
 	if (rec_get_n_fields_old(rec) == DICT_NUM_FIELDS__SYS_INDEXES) {
 		/* MERGE_THRESHOLD exists */
 		field = rec_get_nth_field_old(
@@ -1902,9 +1822,10 @@ err_len:
 
 	field = rec_get_nth_field_old(
 		rec, DICT_FLD__SYS_INDEXES__NAME, &name_len);
-	if (name_len == UNIV_SQL_NULL) {
+	if (name_len == 0 || name_len == UNIV_SQL_NULL) {
 		goto err_len;
 	}
+	ut_ad(field == &rec[8 + 8 + DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN]);
 
 	name_buf = mem_heap_strdupl(heap, (const char*) field,
 				    name_len);
@@ -1932,6 +1853,10 @@ err_len:
 		goto err_len;
 	}
 
+	if (rec_get_deleted_flag(rec, 0)) {
+		return(dict_load_index_del);
+	}
+
 	if (allocate) {
 		*index = dict_mem_index_create(NULL, name_buf, type, n_fields);
 	} else {
@@ -2059,8 +1984,7 @@ dict_load_indexes(
 				ib::warn() << "Failed to load the"
 					" clustered index for table "
 					<< table->name
-					<< " because of the following error: "
-					<< err_msg << "."
+					<< " because of TABLE_ID mismatch."
 					" Refusing to load the rest of the"
 					" indexes (if any) and the whole table"
 					" altogether.";
@@ -2069,8 +1993,13 @@ dict_load_indexes(
 			}
 
 			break;
-		} else if (err_msg == dict_load_index_del) {
-			/* Skip delete-marked records. */
+		}
+
+		if (err_msg == dict_load_index_del) {
+			const trx_id_t id = mach_read_from_6(rec + 8 + 8);
+			if (id > table->def_trx_id) {
+				table->def_trx_id = id;
+			}
 			goto next_rec;
 		} else if (err_msg) {
 			ib::error() << err_msg;
@@ -2079,6 +2008,14 @@ dict_load_indexes(
 			}
 			error = DB_CORRUPTION;
 			goto func_exit;
+		} else if (rec[8 + 8 + DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN]
+			   == static_cast<byte>(*TEMP_INDEX_PREFIX_STR)) {
+			goto next_rec;
+		} else {
+			const trx_id_t id = mach_read_from_6(rec + 8 + 8);
+			if (id > table->def_trx_id) {
+				table->def_trx_id = id;
+			}
 		}
 
 		ut_ad(index);
@@ -2252,6 +2189,12 @@ static const char* dict_load_table_low(const table_name_t& name,
 	(*table)->id = table_id;
 	(*table)->file_unreadable = !!(flags2 & DICT_TF2_DISCARDED);
 
+	ulint len;
+	(*table)->def_trx_id = mach_read_from_6(
+		rec_get_nth_field_old(rec, DICT_FLD__SYS_TABLES__DB_TRX_ID,
+				      &len));
+	ut_ad(len == DATA_TRX_ID_LEN);
+	static_assert(DATA_TRX_ID_LEN == 6, "compatibility");
 	return(NULL);
 }
 
