@@ -48,6 +48,7 @@ Created 3/14/1997 Heikki Tuuri
 #include "handler.h"
 #include "ha_innodb.h"
 #include "fil0fil.h"
+#include <mysql/service_thd_mdl.h>
 
 /*************************************************************************
 IMPORTANT NOTE: Any operation that generates redo MUST check that there
@@ -105,31 +106,102 @@ row_purge_remove_clust_if_poss_low(
 	ulint		mode)	/*!< in: BTR_MODIFY_LEAF or BTR_MODIFY_TREE */
 {
 	dict_index_t* index = dict_table_get_first_index(node->table);
-
-	log_free_check();
-
+	table_id_t table_id = 0;
+	index_id_t index_id = 0;
+	MDL_ticket* mdl_ticket = nullptr;
+	dict_table_t *table = nullptr;
+	pfs_os_file_t f = OS_FILE_CLOSED;
+retry:
+	if (table_id) {
+		dict_sys.mutex_lock();
+		table = dict_table_open_on_id(
+			table_id, true, DICT_TABLE_OP_OPEN_ONLY_IF_CACHED,
+			node->purge_thd, &mdl_ticket);
+		if (!table) {
+			dict_sys.mutex_unlock();
+		} else if (table->n_rec_locks) {
+			for (dict_index_t* ind = UT_LIST_GET_FIRST(
+				     table->indexes); ind;
+			     ind = UT_LIST_GET_NEXT(indexes, ind)) {
+				if (ind->id == index_id) {
+					lock_discard_for_index(*ind);
+				}
+			}
+		}
+	}
 	mtr_t mtr;
 	mtr.start();
 	index->set_modified(mtr);
+	log_free_check();
+	bool success = true;
 
 	if (!row_purge_reposition_pcur(mode, node, &mtr)) {
 		/* The record was already removed. */
+removed:
 		mtr.commit();
-		return true;
+close_and_exit:
+		if (table) {
+			dict_table_close(table, true, false,
+					 node->purge_thd, mdl_ticket);
+			dict_sys.mutex_unlock();
+		}
+		return success;
 	}
 
 	if (node->table->id == DICT_INDEXES_ID) {
 		/* If this is a record of the SYS_INDEXES table, then
 		we have to free the file segments of the index tree
 		associated with the index */
-		dict_drop_index_tree(&node->pcur, nullptr, &mtr);
+		if (!table_id) {
+			const rec_t* rec = btr_pcur_get_rec(&node->pcur);
+
+			table_id = mach_read_from_8(rec);
+			index_id = mach_read_from_8(rec + 8);
+			if (table_id) {
+				mtr.commit();
+				goto retry;
+			}
+			ut_ad("corrupted SYS_INDEXES record" == 0);
+		}
+
+		if (const uint32_t space_id = dict_drop_index_tree(
+			    &node->pcur, nullptr, &mtr)) {
+			if (table) {
+				if (table->release()) {
+					dict_sys.remove(table);
+				} else if (table->space_id == space_id) {
+					table->space = nullptr;
+					table->file_unreadable = true;
+				}
+				table = nullptr;
+				dict_sys.mutex_unlock();
+				if (!mdl_ticket);
+				else if (MDL_context* mdl_context =
+					 static_cast<MDL_context*>(
+						 thd_mdl_context(node->
+								 purge_thd))) {
+					mdl_context->release_lock(mdl_ticket);
+					mdl_ticket = nullptr;
+				}
+			}
+			fil_delete_tablespace(space_id, true, &f);
+		}
+
 		mtr.commit();
+
+		if (table) {
+			dict_table_close(table, true, false,
+					 node->purge_thd, mdl_ticket);
+			dict_sys.mutex_unlock();
+			table = nullptr;
+		}
+
+		purge_sys.check_stop_SYS();
 		mtr.start();
 		index->set_modified(mtr);
 
 		if (!row_purge_reposition_pcur(mode, node, &mtr)) {
-			mtr.commit();
-			return true;
+			goto removed;
 		}
 	}
 
@@ -140,7 +212,6 @@ row_purge_remove_clust_if_poss_low(
 	rec_offs* offsets = rec_get_offsets(rec, index, offsets_,
 					    index->n_core_fields,
 					    ULINT_UNDEFINED, &heap);
-	bool success = true;
 
 	if (node->roll_ptr != row_get_rec_roll_ptr(rec, index, offsets)) {
 		/* Someone else has modified the record later: do not remove */
@@ -185,7 +256,7 @@ func_exit:
 		mtr_commit(&mtr);
 	}
 
-	return(success);
+	goto close_and_exit;
 }
 
 /***********************************************************//**
@@ -662,12 +733,19 @@ row_purge_del_mark(
 	return(row_purge_remove_clust_if_poss(node));
 }
 
+void purge_sys_t::wait_SYS()
+{
+  while (must_wait_SYS())
+    std::this_thread::sleep_for(std::chrono::seconds(1));
+}
+
 /** Reset DB_TRX_ID, DB_ROLL_PTR of a clustered index record
 whose old history can no longer be observed.
 @param[in,out]	node	purge node
 @param[in,out]	mtr	mini-transaction (will be started and committed) */
 static void row_purge_reset_trx_id(purge_node_t* node, mtr_t* mtr)
 {
+retry:
 	/* Reset DB_TRX_ID, DB_ROLL_PTR for old records. */
 	mtr->start();
 
@@ -703,6 +781,17 @@ static void row_purge_reset_trx_id(purge_node_t* node, mtr_t* mtr)
 			ut_ad(!rec_get_deleted_flag(
 					rec, rec_offs_comp(offsets))
 			      || rec_is_alter_metadata(rec, *index));
+			switch (node->table->id) {
+			case DICT_TABLES_ID:
+			case DICT_COLUMNS_ID:
+			case DICT_INDEXES_ID:
+				if (purge_sys.must_wait_SYS()) {
+					mtr->commit();
+					purge_sys.check_stop_SYS();
+					goto retry;
+				}
+			}
+
 			DBUG_LOG("purge", "reset DB_TRX_ID="
 				 << ib::hex(row_get_rec_trx_id(
 						    rec, index, offsets)));
