@@ -52,11 +52,6 @@ Created 12/27/1996 Heikki Tuuri
 #include <algorithm>
 #include <mysql/plugin.h>
 #include <mysql/service_wsrep.h>
-#ifdef WITH_WSREP
-#include "log.h"
-#include "wsrep.h"
-#endif /* WITH_WSREP */
-
 
 /* What kind of latch and lock can we assume when the control comes to
    -------------------------------------------------------------------
@@ -773,7 +768,7 @@ row_upd_build_difference_binary(
 			dfield_t*	vfield = innobase_get_computed_value(
 				update->old_vrow, col, index,
 				&vc.heap, heap, NULL, thd, mysql_table, record,
-				NULL, NULL);
+				NULL, NULL, NULL);
 			if (vfield == NULL) {
 				*error = DB_COMPUTE_VALUE_FAILED;
 				return(NULL);
@@ -1855,7 +1850,8 @@ row_upd_store_v_row(
 							node->row, col, index,
 							&vc.heap, node->heap,
 							NULL, thd, mysql_table,
-							record, NULL, NULL);
+							record, NULL, NULL,
+							NULL);
 					if (vfield == NULL) {
 						return false;
 					}
@@ -2152,30 +2148,34 @@ row_upd_sec_index_entry(
 					err = DB_SUCCESS;
 					break;
 				case DB_LOCK_WAIT:
+					if (UNIV_UNLIKELY(wsrep_debug)) {
+						ib::warn() << "WSREP: sec index FK lock wait"
+							   << " index " << index->name
+							   << " table " << index->table->name
+							   << " query " << wsrep_thd_query(trx->mysql_thd);
+					}
+					break;
 				case DB_DEADLOCK:
-				case DB_LOCK_WAIT_TIMEOUT:
-					WSREP_DEBUG("Foreign key check fail: "
-						"%s on table %s index %s query %s",
-						ut_strerr(err), index->name(), index->table->name.m_name,
-						wsrep_thd_query(trx->mysql_thd));
+					if (UNIV_UNLIKELY(wsrep_debug)) {
+						ib::warn() << "WSREP: sec index FK check fail for deadlock"
+							   << " index " << index->name
+							   << " table " << index->table->name
+							   << " query " << wsrep_thd_query(trx->mysql_thd);
+					}
 					break;
 				default:
-					WSREP_ERROR("Foreign key check fail: "
-						"%s on table %s index %s query %s",
-						ut_strerr(err), index->name(), index->table->name.m_name,
-						wsrep_thd_query(trx->mysql_thd));
+					ib::error() << "WSREP: referenced FK check fail: " << err
+						    << " index " << index->name
+						    << " table " << index->table->name
+						    << " query " << wsrep_thd_query(trx->mysql_thd);
+
 					break;
 				}
 			}
 #endif /* WITH_WSREP */
 		}
 
-#ifdef WITH_WSREP
-		ut_ad(err == DB_SUCCESS || err == DB_LOCK_WAIT
-		      || err == DB_DEADLOCK || err == DB_LOCK_WAIT_TIMEOUT);
-#else
 		ut_ad(err == DB_SUCCESS);
-#endif
 
 		if (referenced) {
 			rec_offs* offsets = rec_get_offsets(
@@ -2496,21 +2496,17 @@ check_fk:
 			case DB_NO_REFERENCED_ROW:
 				err = DB_SUCCESS;
 				break;
-			case DB_LOCK_WAIT:
 			case DB_DEADLOCK:
-			case DB_LOCK_WAIT_TIMEOUT:
-				WSREP_DEBUG("Foreign key check fail: "
-					    "%s on table %s index %s query %s",
-					    ut_strerr(err), index->name(), index->table->name.m_name,
-					    wsrep_thd_query(trx->mysql_thd));
-
+				if (UNIV_UNLIKELY(wsrep_debug)) {
+					ib::warn() << "WSREP: sec index FK check fail for deadlock"
+						   << " index " << index->name
+						   << " table " << index->table->name;
+				}
 				goto err_exit;
 			default:
-				WSREP_ERROR("Foreign key check fail: "
-					    "%s on table %s index %s query %s",
-					    ut_strerr(err), index->name(), index->table->name.m_name,
-					    wsrep_thd_query(trx->mysql_thd));
-
+				ib::error() << "WSREP: referenced FK check fail: " << err
+					    << " index " << index->name
+					    << " table " << index->table->name;
 				goto err_exit;
 			}
 #endif /* WITH_WSREP */
@@ -2617,8 +2613,7 @@ row_upd_clust_rec(
 	the same transaction do not modify the record in the meantime.
 	Therefore we can assert that the restoration of the cursor succeeds. */
 
-	ut_a(btr_pcur_restore_position(BTR_MODIFY_TREE, pcur, mtr) ==
-	    btr_pcur_t::SAME_ALL);
+	ut_a(btr_pcur_restore_position(BTR_MODIFY_TREE, pcur, mtr));
 
 	ut_ad(!rec_get_deleted_flag(btr_pcur_get_rec(pcur),
 				    dict_table_is_comp(index->table)));
@@ -2728,19 +2723,18 @@ row_upd_del_mark_clust_rec(
 		case DB_NO_REFERENCED_ROW:
 			err = DB_SUCCESS;
 			break;
-		case DB_LOCK_WAIT:
 		case DB_DEADLOCK:
-		case DB_LOCK_WAIT_TIMEOUT:
-			WSREP_DEBUG("Foreign key check fail: "
-				    "%d on table %s index %s query %s",
-				    err, index->name(), index->table->name.m_name,
-				    wsrep_thd_query(trx->mysql_thd));
+			if (UNIV_UNLIKELY(wsrep_debug)) {
+				ib::warn() << "WSREP: sec index FK check fail for deadlock"
+					   << " index " << index->name
+					   << " table " << index->table->name;
+			}
 			break;
 		default:
-			WSREP_ERROR("Foreign key check fail: "
-				    "%d on table %s index %s query %s",
-				    err, index->name(), index->table->name.m_name,
-				    wsrep_thd_query(trx->mysql_thd));
+			ib::error() << "WSREP: referenced FK check fail: " << err
+				    << " index " << index->name
+				    << " table " << index->table->name;
+
 			break;
 		}
 #endif /* WITH_WSREP */
@@ -2821,45 +2815,18 @@ row_upd_clust_step(
 		mode = BTR_MODIFY_LEAF;
 	}
 
-	if (btr_pcur_restore_position(mode, pcur, &mtr) !=
-	    btr_pcur_t::SAME_ALL) {
+	if (!btr_pcur_restore_position(mode, pcur, &mtr)) {
 		err = DB_RECORD_NOT_FOUND;
 		goto exit_func;
 	}
 
-	/* If this is a row in SYS_INDEXES table of the data dictionary,
-	then we have to free the file segments of the index tree associated
-	with the index */
-
-	if (node->is_delete == PLAIN_DELETE
-	    && node->table->id == DICT_INDEXES_ID) {
-
-		ut_ad(!dict_index_is_online_ddl(index));
-
-		dict_drop_index_tree(pcur, trx, &mtr);
-
-		mtr.commit();
-
-		mtr.start();
-		index->set_modified(mtr);
-
-		if (btr_pcur_restore_position(BTR_MODIFY_LEAF, pcur, &mtr) !=
-		    btr_pcur_t::SAME_ALL) {
-			err = DB_ERROR;
-
-			mtr.commit();
-
-			return(err);
-		}
-	}
-
 	rec = btr_pcur_get_rec(pcur);
 	offsets = rec_get_offsets(rec, index, offsets_, index->n_core_fields,
 				  ULINT_UNDEFINED, &heap);
 
 	if (!flags && !node->has_clust_rec_x_lock) {
 		err = lock_clust_rec_modify_check_and_lock(
-			0, btr_pcur_get_block(pcur),
+			btr_pcur_get_block(pcur),
 			rec, index, offsets, thr);
 		if (err != DB_SUCCESS) {
 			goto exit_func;
@@ -2868,8 +2835,8 @@ row_upd_clust_step(
 
 	ut_ad(index->table->no_rollback() || index->table->is_temporary()
 	      || row_get_rec_trx_id(rec, index, offsets) == trx->id
-	      || lock_trx_has_expl_x_lock(trx, index->table,
-					  btr_pcur_get_block(pcur),
+	      || lock_trx_has_expl_x_lock(*trx, *index->table,
+					  btr_pcur_get_block(pcur)->page.id(),
 					  page_rec_get_heap_no(rec)));
 
 	if (node->is_delete == PLAIN_DELETE) {
@@ -3090,7 +3057,7 @@ row_upd_step(
 			/* It may be that the current session has not yet
 			started its transaction, or it has been committed: */
 
-			err = lock_table(0, node->table, LOCK_IX, thr);
+			err = lock_table(node->table, LOCK_IX, thr);
 
 			if (err != DB_SUCCESS) {
 
