@@ -1,5 +1,5 @@
 /* Copyright (c) 2000, 2011, Oracle and/or its affiliates.
-   Copyright (c) 2009-2011, Monty Program Ab
+   Copyright (c) 2009, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -175,7 +175,7 @@ static char *backtick_string(CHARSET_INFO *cs, char *to, const char *end,
       last[index]= start;
       index= (index + 1) % 3;
     }
-    char_len= my_charlen_fix(cs, par, par_end);
+    char_len= my_ci_charlen_fix(cs, (const uchar *) par, (const uchar *) par_end);
     if (char_len == 1 && c == (uchar) quote_char )
     {
       if (start + 1 >= end)
@@ -224,48 +224,87 @@ err:
 */
 
 static char *process_str_arg(CHARSET_INFO *cs, char *to, const char *end,
-                             size_t width, char *par, uint print_type)
+                             longlong length_arg, size_t width, char *par,
+                             uint print_type, my_bool nice_cut)
 {
   int well_formed_error;
   uint dots= 0;
   size_t plen, left_len= (size_t) (end - to) + 1, slen=0;
+  my_bool left_fill= 1;
+  size_t length;
+
+  /*
+    The sign of the length argument specific the string should be right
+    or left adjusted
+  */
+  if (length_arg < 0)
+  {
+    length= (size_t) -length_arg;
+    left_fill= 0;
+  }
+  else
+    length= (size_t) length_arg;
+
   if (!par)
     par = (char*) "(null)";
 
-  plen= slen= strnlen(par, width + 1);
-  if (plen > width)
-    plen= width;
-  if (left_len <= plen)
-    plen = left_len - 1;
-  if ((slen > plen))
+  if (nice_cut)
   {
-    if (plen < 3)
+    plen= slen= strnlen(par, width + 1);
+    if (plen > width)
+      plen= width;
+    if (left_len <= plen)
     {
-      dots= (uint) plen;
-      plen= 0;
+      plen = left_len - 1;
+      length= plen;
     }
-    else
+    if ((slen > plen))
     {
-      dots= 3;
-      plen-= 3;
+      if (plen < 3)
+      {
+        dots= (uint) plen;
+        plen= 0;
+      }
+      else
+      {
+        dots= 3;
+        plen-= 3;
+      }
+    }
+  }
+  else
+  {
+    plen= slen= strnlen(par, width);
+    dots= 0;
+    if (left_len <= plen)
+    {
+      plen = left_len - 1;
+      length= plen;
     }
   }
 
   plen= my_well_formed_length(cs, par, par + plen, width, &well_formed_error);
   if (print_type & ESCAPED_ARG)
   {
+    const char *org_to= to;
     to= backtick_string(cs, to, end, par, plen + dots, '`', MY_TEST(dots));
+    plen= (size_t) (to - org_to);
     dots= 0;
   }
   else
-    to= strnmov(to,par,plen);
-
-  if (dots)
   {
-    for (; dots; dots--)
-      *(to++)= '.';
-    *(to)= 0;
+    if (left_fill)
+    {
+      if (plen + dots < length)
+        to= strfill(to, length - plen - dots, ' ');
+    }
+    to= strnmov(to,par,plen);
+    if (dots)
+      to= strfill(to, dots, '.');
   }
+
+  if (!left_fill && plen + dots < length)
+    to= strfill(to, length - plen - dots, ' ');
   return to;
 }
 
@@ -368,7 +407,7 @@ static char *process_int_arg(char *to, const char *end, size_t length,
 
 
 /**
-  Procesed positional arguments.
+  Processed positional arguments.
 
   @param cs         string charset
   @param to         buffer where processed string will be place
@@ -446,6 +485,7 @@ start:
       switch (args_arr[i].arg_type) {
       case 's':
       case 'b':
+      case 'T':
         args_arr[i].str_arg= va_arg(ap, char *);
         break;
       case 'f':
@@ -480,12 +520,19 @@ start:
       size_t width= 0, length= 0;
       switch (print_arr[i].arg_type) {
       case 's':
+      case 'T':
       {
+        longlong min_field_width;
         char *par= args_arr[print_arr[i].arg_idx].str_arg;
         width= (print_arr[i].flags & WIDTH_ARG)
           ? (size_t)args_arr[print_arr[i].width].longlong_arg
           : print_arr[i].width;
-        to= process_str_arg(cs, to, end, width, par, print_arr[i].flags);
+        min_field_width= (print_arr[i].flags & LENGTH_ARG)
+          ? args_arr[print_arr[i].length].longlong_arg
+          : (longlong) print_arr[i].length;
+        to= process_str_arg(cs, to, end, min_field_width, width, par,
+                            print_arr[i].flags,
+                            (print_arr[i].arg_type == 'T'));
         break;
       }
       case 'b':
@@ -551,8 +598,8 @@ start:
           *to++= ' ';
           *to++= '"';
           my_strerror(errmsg_buff, sizeof(errmsg_buff), (int) larg);
-          to= process_str_arg(cs, to, real_end, width, errmsg_buff,
-                              print_arr[i].flags);
+          to= process_str_arg(cs, to, real_end, 0, width, errmsg_buff,
+                              print_arr[i].flags, 1);
           if (real_end > to) *to++= '"';
         }
         break;
@@ -676,10 +723,11 @@ size_t my_vsnprintf_ex(CHARSET_INFO *cs, char *to, size_t n,
 
     fmt= check_longlong(fmt, &have_longlong);
 
-    if (*fmt == 's')				/* String parameter */
+    if (*fmt == 's' || *fmt == 'T')			/* String parameter */
     {
       reg2 char *par= va_arg(ap, char *);
-      to= process_str_arg(cs, to, end, width, par, print_type);
+      to= process_str_arg(cs, to, end, (longlong) length, width, par,
+                          print_type, (*fmt == 'T'));
       continue;
     }
     else if (*fmt == 'b')				/* Buffer parameter */
@@ -690,7 +738,13 @@ size_t my_vsnprintf_ex(CHARSET_INFO *cs, char *to, size_t n,
     }
     else if (*fmt == 'f' || *fmt == 'g')
     {
+#if __has_feature(memory_sanitizer)         /* QQ: MSAN has double trouble? */
+      __msan_check_mem_is_initialized(ap, sizeof(double));
+#endif
       double d= va_arg(ap, double);
+#if __has_feature(memory_sanitizer)        /* QQ: MSAN has double trouble? */
+      __msan_unpoison(&d, sizeof(double));
+#endif
       to= process_dbl_arg(to, end, width, d, *fmt);
       continue;
     }
@@ -731,7 +785,8 @@ size_t my_vsnprintf_ex(CHARSET_INFO *cs, char *to, size_t n,
         *to++= ' ';
         *to++= '"';
         my_strerror(errmsg_buff, sizeof(errmsg_buff), (int) larg);
-        to= process_str_arg(cs, to, real_end, width, errmsg_buff, print_type);
+        to= process_str_arg(cs, to, real_end, 0, width, errmsg_buff,
+                            print_type, 1);
         if (real_end > to) *to++= '"';
       }
       continue;
@@ -815,7 +870,7 @@ int my_vfprintf(FILE *stream, const char* format, va_list args)
     if (new_len < cur_len)
       return 0;                                 /* Overflow */
     cur_len= new_len;
-    p= my_malloc(cur_len, MYF(MY_FAE));
+    p= my_malloc(PSI_INSTRUMENT_ME, cur_len, MYF(MY_FAE));
     if (!p)
       return 0;
   }
